/*********************************************
/ BotBehaviors.cpp
/
/ This file contains code for handling external
/ behavior modules.
*********************************************/

#include "extdll.h" 
#include "util.h"
#include "cbase.h"
#include "bot.h"

#include "FlexStat.h"
#include "SocketClient.h"

// extern SocketClient statSocket;
extern FlexStat * flexStat;

/*********************************************
/ UNLOAD BEHAVIOR DLL
/ Unloads the bot's behavior DLL.
*********************************************/
void CBot::UnloadBehaviorDll()
{
	if (m_hInstDll)
	{
		if (m_pfnRemoveBotBehavior)
			(m_pfnRemoveBotBehavior)(m_behaviorInstance);
		FreeLibrary(m_hInstDll);
		m_pfnCreateBotBehavior = NULL;
		m_pfnRemoveBotBehavior = NULL;
		botTask = DO_NOTHING;
		CleanupDebugBeams();
	}
}

/*********************************************
/ LOAD BEHAVIOR DLL
/ Does the work to load a behvior DLL with the
/ given filename. Returns true if loaded successfully
/ and false if there is an error.
*********************************************/
bool CBot::LoadBehaviorDll(const char * dllFilename)
{
	// Unload the old behavior DLL
	UnloadBehaviorDll();

	// Get the path of the new DLL to load
	char dllPath[1000];
	BuildModFilename(dllPath, "dlls/behavior_dlls/");
	strcat(dllPath, dllFilename);
	
	// Load the DLL
	m_hInstDll = LoadLibrary(dllPath);

	// Could the DLL be loaded?
	if (m_hInstDll)
	{
		bool * usesNewTurningValues = (bool *) GetProcAddress(m_hInstDll, "UsesNewTurningValues");
		if (!usesNewTurningValues) {
			// MessageBox(NULL, "The user DID NOT SPECIFY the turning style. The old turning style will be used.", "Turning style...", MB_OK);
			newTurnSignsP = false;

		}
		else if (*usesNewTurningValues == false) {
			// MessageBox(NULL, "The user chose NOT to use the new turning style. The old turning style will be used.", "Turning style...", MB_OK);
			newTurnSignsP = false;
		}
		else {
			// MessageBox(NULL, "The user chose to use the new turning style. The new turning style will be used.", "Turning style...", MB_OK);
			newTurnSignsP = true;
		}

		m_pfnCreateBotBehavior = (PFN_CREATE_BOT_BEHAVIOR) GetProcAddress(m_hInstDll, "CreateBotBehavior");
		if (!m_pfnCreateBotBehavior)
		{
			BotDebug("Could not load pointer to CreateBotBehavior()\n");
			return FALSE;
		}
		m_pfnRemoveBotBehavior = (PFN_REMOVE_BOT_BEHAVIOR) GetProcAddress(m_hInstDll, "RemoveBotBehavior");
		if (!m_pfnCreateBotBehavior)
		{
			BotDebug("Could not load pointer to RemoveBotBehavior()\n");
			return FALSE;
		}
		m_pfnDoExternalBotThink = (DO_BOT_THINK) GetProcAddress(m_hInstDll, "Think");
		if (!m_pfnDoExternalBotThink)
		{
			BotDebug("Could not load pointer to Think()\n");
			return FALSE;
		}
		m_pfnGetExternalRotate = (GET_ACTION_INTEGER) GetProcAddress(m_hInstDll, "GetRotate");
		if (!m_pfnGetExternalRotate)
		{
			BotDebug("Could not load pointer to GetRotate()\n");
			return FALSE;
		}
		m_pfnGetExternalTranslate = (GET_ACTION_INTEGER) GetProcAddress(m_hInstDll, "GetTranslate");
		if (!m_pfnGetExternalTranslate)
		{
			BotDebug("Could not load pointer to GetTranslate()\n");
			return FALSE;
		}
		m_pfnGetExternalStrafe = (GET_ACTION_INTEGER) GetProcAddress(m_hInstDll, "GetStrafe");
		if (!m_pfnGetExternalStrafe)
		{
			BotDebug("Could not load pointer to GetStrafe()\n");
			return FALSE;
		}
		m_pfnGetExternalPitch = (GET_ACTION_INTEGER) GetProcAddress(m_hInstDll, "GetPitch");
		if (!m_pfnGetExternalPitch)
		{
			BotDebug("Could not load pointer to GetPitch()\n");
			return FALSE;
		}
		m_pfnGetExternalShoot = (GET_ACTION_BOOLEAN) GetProcAddress(m_hInstDll, "GetShoot");
		if (!m_pfnGetExternalShoot)
		{
			BotDebug("Could not load pointer to GetShoot()\n");
			return FALSE;
		}
		m_pfnGetExternalJump = (GET_ACTION_BOOLEAN) GetProcAddress(m_hInstDll, "GetJump");
		if (!m_pfnGetExternalJump)
		{
			BotDebug("Could not load pointer to GetJump()\n");
			return FALSE;
		}
		m_pfnGetExternalDuck = (GET_ACTION_BOOLEAN) GetProcAddress(m_hInstDll, "GetDuck");
		if (!m_pfnGetExternalDuck)
		{
			BotDebug("Could not load pointer to GetDuck()\n");
			return FALSE;
		}
		m_pfnGetExternalSecondary = (GET_ACTION_BOOLEAN) GetProcAddress(m_hInstDll, "GetSecondShoot");
		if (!m_pfnGetExternalSecondary)
		{
			BotDebug("Could not load pointer to GetSecondShoot()\n");
			return FALSE;
		}
		m_pfnGetExternalWeapon = (GET_ACTION_INTEGER) GetProcAddress(m_hInstDll, "GetWeapon");
		if (!m_pfnGetExternalWeapon)
		{
			BotDebug("Could not load pointer to GetWeapon()\n");
			return FALSE;
		}
		m_pfnGetExternalReload = (GET_ACTION_BOOLEAN) GetProcAddress(m_hInstDll, "GetReload");
		if (!m_pfnGetExternalReload)
		{
			BotDebug("Could not load pointer to GetReload()\n");
			return FALSE;
		}
		m_pfnGetExternalUse = (GET_ACTION_BOOLEAN) GetProcAddress(m_hInstDll, "GetUse");
		if (!m_pfnGetExternalUse)
		{
			BotDebug("Could not load pointer to GetUse()\n");
			return FALSE;
		}

		// Create a bot brain
		m_behaviorInstance = (int)(m_pfnCreateBotBehavior)();
		if (m_behaviorInstance < 0)
		{
			BotDebug("Failure was returned by a successfully-loaded CreateBotBehavior()\n");
			return FALSE;
		}

		// Initialize performance metrics
		totalThinkTime = 0;
		totalThinks = 0;

		// Store the behavior name
		strcpy(behaviorName, dllFilename);
		if (!flexStat)
			flexStat = new FlexStat;
		flexStat->UpdateBehavior(playerNum, behaviorName);

		/* Notify the stats server of the new behavior
		//this was commented out to avoid a sendmessage slow down when no server is preset :safe
		char statMsg[1000];
		sprintf(statMsg, "behavior %d %s\n", playerNum, behaviorName);
		statSocket.SendMessage(statMsg);
		*/
		return TRUE;
	}
	else
	{
		// Return failure
		BotDebug("BEHAVIOR LOADING ERROR: File either does not exist or is not a DLL.\n");
		return FALSE;
	}
}