/*********************************************
/ BotCommands.cpp
/
/ This file contains code to handle commands that
/ are given to the bots via the BotMessenger
/ infrastructure.
*********************************************/

#include "extdll.h" 
#include "util.h"
#include "cbase.h"
#include "bot.h"

#include "BotMessenger.h"

#define MAX_ARGS 50
#define MAX_CMD_LENGTH 500

extern CBotMessenger botMessenger;

/*********************************************
/ HANDLE BOT COMMANDS
/ Checks the bot message queue to see if the bot
/ has a new message. If there is a new message,
/ respond accordingly.
*********************************************/
void CBot::HandleBotCommands()
{
	// Check if there is a message waiting for the bot
	CBotMessage * message = botMessenger.GetBotMessage(playerNum);
	
	// If there is a message, check to see if is is valid and handle it
	if (message) {
		
		char * argv[MAX_ARGS];
		int argc = 0;
		char * token;
		
		// Get a copy of the argument string because strtok is destructive
		char argcpy[MAX_CMD_LENGTH];
		strcpy(argcpy, message->msgArg);
		
		// Parse the arguments into an array
		token = strtok(argcpy, " ");
		while (token != NULL)
		{
			argv[argc] = token;
			++argc;
			token = strtok(NULL, " ");
		}
		
		// Pause the bot
		if (FStrEq(message->msgText, "pause"))
		{
			if (m_bDebugOn)
			{
				sprintf(m_debugStr, "Player %d received a 'pause' command\n", playerNum, message->msgFromPlayer);
				BotDebug(m_debugStr);
			}
			if (argc >= 1)
			{
				if (FStrEq(argv[0], "on"))
					m_bPaused = TRUE;
				else if (FStrEq(argv[0], "off"))
					m_bPaused = FALSE;
				else if (FStrEq(argv[0], "*none*"))
					m_bPaused = toggle(m_bPaused);
			}
			else
				// Toggle the pause status
				m_bPaused = toggle(m_bPaused);
		}
		// Get performance metrics
		else if (FStrEq(message->msgText, "metrics"))
		{
			sprintf(m_debugStr, "Average milliseconds per think cycle: %f\n", ((float)totalThinkTime)/totalThinks);
			BotDebug(m_debugStr);
		}
		/*	else if (FStrEq(message->msgText, "unload"))
		{
		UnloadBehaviorDll();
		sprintf(m_debugStr, "Bot %d has unloaded its behavior DLL\n", playerNum);
		BotDebug(m_debugStr);
	} */
		// Set the accuracy of the bot
		else if (FStrEq(message->msgText, "skill"))
		{
			if (argc == 1)
			{
				if (atoi(argv[0])>=0 && atoi(argv[0])<=100)
				{
					aimSkill = atoi(argv[0]);
					sprintf(m_debugStr, "Bot %d skill has been set to %d\n", playerNum, aimSkill);
					BotDebug(m_debugStr);
				}
				else
					BotDebug("Skill must be a value between 0 (terrible) and 100 (perfect).\n");
			}
			else
				BotDebug("Skill setting must specify one value between 0 (terrible) and 100 (perfect).\n");
		}
		// Load a dll message
		else if (FStrEq(message->msgText, "dll"))
		{
			if (m_bDebugOn)
			{
				sprintf(m_debugStr, "Player %d received a 'dll' command (%s)\n", playerNum, message->msgArg, message->msgFromPlayer);
				BotDebug(m_debugStr);
			}
			// Load the behavior DLL
			if (LoadBehaviorDll(message->msgArg))
				botTask = DLL;
			else
				botTask = DO_NOTHING;
		}
		// Debug toggle message
		else if (FStrEq(message->msgText, "debug"))
		{
			if (m_bDebugOn)
			{
				sprintf(m_debugStr, "Player %d received a 'debug' command from player %d\n", playerNum, message->msgFromPlayer);
				BotDebug(m_debugStr);
			}
			
			if (argc > 0)
			{
				if (FStrEq(argv[0], "on"))
					m_bDebugOn = TRUE;
				else if (FStrEq(argv[0], "off"))
					m_bDebugOn = FALSE;
			}
			else
				m_bDebugOn = toggle(m_bDebugOn);
			
			// Do cleanup of vision beams
			if (!m_bDebugOn)
				CleanupDebugBeams();
		}
		else if (FStrEq(message->msgText, "autodebug"))
		{
			if (m_bDebugOn)
			{
				sprintf(m_debugStr, "Player %d received a 'debug' command from player %d\n", playerNum, message->msgFromPlayer);
				BotDebug(m_debugStr);
			}
			if (argc > 0)
			{
				if (FStrEq(argv[0], "on"))
					m_bAutoSensorDebug = TRUE;
				else if (FStrEq(argv[0], "off"))
					m_bAutoSensorDebug = FALSE;
			}
			else
				m_bAutoSensorDebug = toggle(m_bAutoSensorDebug);
			// Do cleanup of vision beams
			if (!m_bDebugOn)
				CleanupDebugBeams();
		}
		else if (FStrEq(message->msgText, "waypoint"))
		{
			if (argc > 0)
			{
				if(FStrEq(argv[0], "add"))
				{
					CBaseEntity * Player = UTIL_PlayerByIndex(message->msgFromPlayer);
					//parse the coordenates out of the message->msgArg
					char Xcoord[64];
					sprintf(Xcoord, "%f", Player->pev->origin.x);
					char Ycoord[64];
					sprintf(Ycoord, "%f", Player->pev->origin.y);
					char Zcoord[64];
					sprintf(Zcoord, "%f", Player->pev->origin.z);
					if(Xcoord == NULL || Ycoord == NULL || Zcoord == NULL)
					{
						//error of some bizarre kind
						return;
					}
					WAYPOINT botWP;
					botWP.origin = Vector(atoi(Xcoord), atoi(Ycoord) , atoi(Zcoord) -34 );
					botWP.type = WAYPOINT_CHOKEPOINT;
					AddWaypoint(botWP);
					//print a return
					sprintf(m_debugStr, "player %d received a waypoint add command\n", playerNum, message->msgArg, message->msgFromPlayer);
					BotDebug(m_debugStr);
				}
				else if(FStrEq(argv[0], "delete"))
				{
					//make sure that there is something to delete
					if(m_nWaypoints == 0){
						sprintf(m_debugStr, "delete command not done; no waypoints to delete", playerNum, message->msgFromPlayer);
						BotDebug(m_debugStr);
						return;
					}
					
					CBaseEntity * Player = UTIL_PlayerByIndex(message->msgFromPlayer);
					//parse the coordenates out of the message->msgArg
					char Xcoord[64];
					sprintf(Xcoord, "%f", Player->pev->origin.x);
					char Ycoord[64];
					sprintf(Ycoord, "%f", Player->pev->origin.y);
					char Zcoord[64];
					sprintf(Zcoord, "%f", Player->pev->origin.z);
					Vector PlayerPos = Vector(atoi(Xcoord), atoi(Ycoord) , atoi(Zcoord));
					
					int nearestWaypoint = -1;
					int index;
					for(index = 0; index < m_nWaypoints ; index++)
					{
						if(waypoints[index].removed == FALSE){
							if(nearestWaypoint == -1)
								nearestWaypoint = index;
							Vector toWP = waypoints[index].waypoint.origin - PlayerPos;
							Vector nearestWP = waypoints[nearestWaypoint].waypoint.origin - PlayerPos;
							if(toWP.Length() < nearestWP.Length())
								nearestWaypoint = index;
						}
					}
					
					if(nearestWaypoint != -1)
					{
						DelWaypoint(waypoints[nearestWaypoint]);
						sprintf(m_debugStr, "player %d received a waypoint delete command; the nearest waypoint was deleted\n", playerNum, index, message->msgFromPlayer);
						BotDebug(m_debugStr);
					}
					else
					{
						sprintf(m_debugStr, "waypoint delete command not done; no more waypoints to delete\n", playerNum, index, message->msgFromPlayer);
						BotDebug(m_debugStr);
					}
				}
				else if (FStrEq(argv[0], "export"))
				{
					if (argc < 2)
					{
						BotDebug("Correct usage: bot n waypoint export [filename]\n");
						return;
					}
					ExportWaypointsToFile(argv[1]);
				}
				else if (FStrEq(argv[0], "load"))
				{
					if (argc < 2)
					{
						BotDebug("Correct usage: bot n waypoint load [filename]\n");
						return;
					}
					ReadWaypointsFromFile(argv[1]);
				}
				else
				{
					sprintf(m_debugStr, "To use waypoints you must say weather to add or delete\n-add: creates a waypoint where you are standing\n-delete: destroys the nearest waypoint to you\n", playerNum, message->msgFromPlayer);
					BotDebug(m_debugStr);
				}
			}
			
		}
		else if (FStrEq(message->msgText, "infopoint"))
		{
			if(argc < 2){
				sprintf(m_debugStr, "To use infopoints you must say weather to add or delete and of what type:\n add - bot 2(bot) infopoint add 30(time) goto(type) \n delete - bot 2(bot) infopoint delete goto(type)\n", playerNum, message->msgFromPlayer);
				BotDebug(m_debugStr);
				return;
			}
			if(FStrEq(argv[0], "add"))
			{
				if (argc < 3)
				{
					sprintf(m_debugStr, "To use infopoint add you must provide a time to last and a type; goto, enemy, or camping", playerNum, message->msgFromPlayer);
					BotDebug(m_debugStr);
					return;
				}
				CBaseEntity * Player = UTIL_PlayerByIndex(message->msgFromPlayer);
				//parse the coordenates out of the message->msgArg
				char Xcoord[64];
				sprintf(Xcoord, "%f", Player->pev->origin.x);
				char Ycoord[64];
				sprintf(Ycoord, "%f", Player->pev->origin.y);
				char Zcoord[64];
				sprintf(Zcoord, "%f", Player->pev->origin.z);
				if(Xcoord == NULL || Ycoord == NULL || Zcoord == NULL)
				{
					//error of some kind
					return;
				}
				WAYPOINT botWP;
				botWP.origin = Vector(atoi(Xcoord), atoi(Ycoord) , atoi(Zcoord) -34 );
				int lifeTime = atoi(argv[2]);
				
				//BotDebug(IPtype);

				//sprintf(IPtype, "%", (int)argv[2]);
				botWP.type = atoi(argv[1]);

				AddInfopoint(botWP, gpGlobals->time, lifeTime);
				//print a return to console
				sprintf(m_debugStr, "player %d received a infopoint add command\n", playerNum);
				BotDebug(m_debugStr);
			}
			else if(FStrEq(argv[0], "delete"))
			{
				CBaseEntity * Player = UTIL_PlayerByIndex(message->msgFromPlayer);
				//parse the coordenates out of the message->msgArg
				char Xcoord[64];
				sprintf(Xcoord, "%f", Player->pev->origin.x);
				char Ycoord[64];
				sprintf(Ycoord, "%f", Player->pev->origin.y);
				char Zcoord[64];
				sprintf(Zcoord, "%f", Player->pev->origin.z);
				Vector PlayerPos = Vector(atoi(Xcoord), atoi(Ycoord) , atoi(Zcoord));
				
				int nearestWaypoint = -1;
				int index;
				int remove_type = atoi(argv[1]);
				if(m_nInfopoints == 0)
				{
					sprintf(m_debugStr, "delete command not done; no infopoints to delete\n");
					BotDebug(m_debugStr);
					return;
				}
				for(index = 0; index < m_nInfopoints ; index++)
				{
					if(Infopoints[index].removed == FALSE)
						if(Infopoints[index].waypoint.type == remove_type)//check type
						{ //found one of the right type
							if(nearestWaypoint == -1)
								nearestWaypoint = index;
							Vector toIP = Infopoints[index].waypoint.origin - PlayerPos;
							Vector nearestIP = Infopoints[nearestWaypoint].waypoint.origin - PlayerPos;
							if(toIP.Length() < nearestIP.Length())
								nearestWaypoint = index;
						}
						
				}
				if(nearestWaypoint != -1){
					expireInfopoint(Infopoints[nearestWaypoint]);
					sprintf(m_debugStr, "player %d received a delete infopoint command of type, the nearest infopoint was deleted\n", playerNum, remove_type);
					BotDebug(m_debugStr);
				}else{
					sprintf(m_debugStr, "delete enemy infopoint not done; no more infopoints of type %d to delete\n", playerNum, index, message->msgFromPlayer, remove_type);
					BotDebug(m_debugStr);
				}
			}else{
				sprintf(m_debugStr, "To use infopoint delete you must provide a type that has been used");
				BotDebug(m_debugStr);
			}
		}
		else
		{
			if (m_bDebugOn)
			{
				sprintf(m_debugStr, "UNRECOGNIZED MESSAGE: Player %d received a '%s' command (%s) from player %d\n", playerNum, message->msgText, message->msgArg, message->msgFromPlayer);
				BotDebug(m_debugStr);
				//}else{
				//BotDebug("not on the hole good\n");
				// }
			}
		}
	}
  delete message;
}
