/*********************************************
/ BotExports.cpp
/
/ This file contains the DLL-exported wrapper calls to
/ bot sensors. These wrappers are required so that the
/ behavior DLL is independent of the bot implementation.
/ They are called on the appropriate bot by using the
/ pThinkingBot variable to keep track of which bot is
/ currently in its think cycle.
*********************************************/

#include "extdll.h" 
#include "util.h"
#include "cbase.h"
#include "bot.h"
#include "ExternalBotDebug.h"

#include <windows.h>

#include "BotExports.h"

//********************************************************
// If the think algorithm is changed to be threaded, 
// this must be changed (only works for linear "thinking")*/
CBot * pThinkingBot;	// pointer to the bot who is currently thinking
//*******************************************************/

char debugStr[256];

/*********************************************
/ DEBUG SIGNAL WRAPPER
*********************************************/
void EXPORT BOTEXP_DebugSignal(const char * message)
{
	SendExternalDebug(pThinkingBot->playerNum, (char *)message);
}

/*********************************************
/ GET HEALTH WRAPPER
*********************************************/
int EXPORT BOTEXP_GetHealth()
{
	int health = pThinkingBot->get_health();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "health %d", health);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return health;
}

/*********************************************
/ GET YAW WRAPPER
*********************************************/
float EXPORT BOTEXP_GetYaw()
{
	float yaw = pThinkingBot->get_yaw();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "yaw %d", yaw);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return yaw;
}

/*********************************************
/ GET PITCH WRAPPER
*********************************************/
float EXPORT BOTEXP_GetPitch()
{
	float pitch = pThinkingBot->get_pitch();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "pitch %d", pitch);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return pitch;
}

/*********************************************
/ NEAREST VISIBLE ENEMY NUMBER WRAPPER
*********************************************/
int EXPORT BOTEXP_NearestVisEnemyNum()
{
	int playerNum = pThinkingBot->nearest_vis_enemy_num();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "nearest-visible-enemy %d", playerNum);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return playerNum;
}

/*********************************************
/ GET VISION DISTANCE WRAPPER
*********************************************/
int EXPORT BOTEXP_GetVisionDistance(float angle, int range)
{
	int dist = pThinkingBot->get_vision_distance(angle, range);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "vision-distance-%d-deg %d", angle, dist);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return dist;
}

/*********************************************
/ GET VISION DISTANCE WRAPPER
*********************************************/
int EXPORT BOTEXP_GetVisionDistanceWithPitch(float angle,float pitch, int range)
{
	int dist = pThinkingBot->get_vision_distance(angle, pitch, range);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "vision-dist-%d-deg-%d-pitch %d", angle, pitch, dist);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return dist;
}

/*********************************************
/ GET VISION SWEEP DISTANCE WRAPPER
*********************************************/
int EXPORT BOTEXP_GetVisionSweepDistance(float angle, int range, int sweep, int interval)
{
	int dist = pThinkingBot->get_vision_sweep_distance(angle, range, sweep, interval);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "vision-dist-%d-deg-%d %d", angle, range, dist);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return dist;
}

/*********************************************
/ GET LEFT DISTANCE WRAPPER
*********************************************/
int EXPORT BOTEXP_GetLeftDistance(int range)
{
	int dist = pThinkingBot->get_left_distance(range);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "left-distance %d", dist);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return dist;
}

/*********************************************
/ GET RIGHT DISTANCE WRAPPER
*********************************************/
int EXPORT BOTEXP_GetRightDistance(int range)
{
	int dist = pThinkingBot->get_right_distance(range);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "right-distance %d", dist);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return dist;
}

/*********************************************
/ GET CENTER DISTANCE WRAPPER
*********************************************/
int EXPORT BOTEXP_GetCenterDistance(int range)
{
	int dist = pThinkingBot->get_center_distance(range);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "center-distance %d", dist);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return dist;
}

/*********************************************
/ YAW TO PLAYER WRAPPER
*********************************************/
float EXPORT BOTEXP_YawToPlayer(int playerNum)
{
	float yaw = pThinkingBot->yaw_to_player(playerNum);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "yaw-to-player-%d %f", playerNum, yaw);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return yaw;
}

/*********************************************
/ PITCH TO PLAYER WRAPPER
*********************************************/
float EXPORT BOTEXP_PitchToPlayer(int playerNum)
{
	float pitch = pThinkingBot->pitch_to_player(playerNum);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "pitch-to-player-%d %f", playerNum, pitch);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return pitch;
}

/*********************************************
/ DISTANCE TO PLAYER WRAPPER
*********************************************/
int EXPORT BOTEXP_DistanceToPlayer(int playerNum)
{
	int dist = pThinkingBot->distance_to_player(playerNum);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "distance-to-player-%d %d", playerNum, dist);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return dist;
}

/*********************************************
/ IS PLAYER VISIBLE WRAPPER
*********************************************/
bool EXPORT BOTEXP_IsPlayerVisible(int playerNum)
{
	bool visible = pThinkingBot->is_player_visible(playerNum);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		if (visible)
			sprintf(debugStr, "is-player-%d-visible TRUE", playerNum);
		else
			sprintf(debugStr, "is-player-%d-visible FALSE", playerNum);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return visible;
}
/*********************************************
/ IS MOVING WRAPPER
*********************************************/
bool EXPORT BOTEXP_IsMoving()
{
	bool moving = pThinkingBot->is_moving();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		if (moving)
			sprintf(debugStr, "is-moving TRUE");
		else
			sprintf(debugStr, "is-moving FALSE");
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return moving;
}

/*********************************************
/ YAW TO WAYPOINT WRAPPER
*********************************************/

float EXPORT BOTEXP_YawToWaypoint(int waypointIndex)
{
	float yaw = pThinkingBot->yaw_to_waypoint(waypointIndex);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "yaw-to-waypoint-%d %f", waypointIndex, yaw);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return yaw;
}

/*********************************************
/ DISTANCE TO WAYPOINT WRAPPER
*********************************************/

int EXPORT BOTEXP_DistanceToWaypoint(int waypointIndex)
{
	int dist = pThinkingBot->distance_to_waypoint(waypointIndex);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "distance-to-waypoint-%d %d", waypointIndex, dist);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return dist;
}

/*********************************************
/ NEAREST WAYPOINT NUMBER WRAPPER
*********************************************/
int EXPORT BOTEXP_NearestWaypoint()
{
	int waypointNum = pThinkingBot->nearest_waypoint_num();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "nearest-waypoint %d", waypointNum);
		SendExternalDebug(waypointNum, debugStr);
	}
	return waypointNum;
}

/*********************************************
/ YAW TO INFOPOINT WRAPPER
*********************************************/

float EXPORT BOTEXP_YawToInfopoint(int infopointIndex, int type)
{
	float yaw = pThinkingBot->yaw_to_infopoint(infopointIndex, type);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "yaw-to-infopoint-%d %f", infopointIndex, yaw);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return yaw;
}

/*********************************************
/ DISTANCE TO INFOPOINT WRAPPER
*********************************************/

int EXPORT BOTEXP_DistanceToInfopoint(int infopointIndex, int type)
{
	int dist = pThinkingBot->distance_to_infopoint(infopointIndex, type);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "distance-to-infopoint-%d %d", infopointIndex, dist);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return dist;
}

/*********************************************
/ NEAREST INFOPOINT NUMBER WRAPPER
*********************************************/
int EXPORT BOTEXP_NearestInfopoint(int type)
{
	int infopointNum = pThinkingBot->nearest_infopoint_num(type);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "nearest-infopoint %d", infopointNum);
		SendExternalDebug(infopointNum, debugStr);
	}
	return infopointNum;
}

/*********************************************
/ NEXT IP TO EXPIRE NUMBER WRAPPER
*********************************************/
int EXPORT BOTEXP_NextIPToExpire(int type)
{
	int infopointNum = pThinkingBot->next_ip_to_expire_num(type);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "next_infopoint_to_expire %d", infopointNum);
		SendExternalDebug(infopointNum, debugStr);
	}
	return infopointNum;
}

/*********************************************
/ AMMO IN CLIP
*********************************************/
int EXPORT BOTEXP_AmmoInClip()
{
	int rounds = pThinkingBot->ammo_in_clip();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "ammo_in_clip %d", rounds);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return rounds;
}

/*********************************************
/ NEAREST VISIBLE TEAMMATE NUMBER WRAPPER
*********************************************/
int EXPORT BOTEXP_NearestVisTeammateNum()
{
	int playerNum = pThinkingBot->nearest_vis_teammate_num();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "nearest-visible-enemy %d", playerNum);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return playerNum;
}

/*********************************************
/ YAW TO ENEMY SOUND WRAPPER
*********************************************/
float EXPORT BOTEXP_YawToEnemySound()
{
	float yaw_to_sound = pThinkingBot->yaw_to_enemy_sound();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "yaw-to-enemy-sound %d", yaw_to_sound);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return yaw_to_sound;
}

/*********************************************
/ DISTANCE TO ENEMY SOUND WRAPPER
*********************************************/
int EXPORT BOTEXP_DistanceToEnemySound()
{
	int distance_to_sound = pThinkingBot->distance_to_enemy_sound();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "distance-to-enemy-sound %d", distance_to_sound);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return distance_to_sound;
}

/*********************************************
/ NEAREST ITEM WRAPPER
*********************************************/
int EXPORT BOTEXP_NearestItem(int itemType)
{
	int nearest_item = pThinkingBot->nearest_item(itemType);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "nearest-item-type-%d %d", itemType, nearest_item);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return nearest_item;
}

/*********************************************
/ YAW TO ITEM WRAPPER
*********************************************/
float EXPORT BOTEXP_YawToItem(int itemType, int index)
{
	float yaw = pThinkingBot->yaw_to_item(itemType, index);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "yaw-to-item %d", yaw);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return yaw;
}

/*********************************************
/ DISTANCE TO ITEM WRAPPER
*********************************************/
int EXPORT BOTEXP_DistanceToItem(int itemType, int index)
{
	int distance = pThinkingBot->distance_to_item(itemType, index);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "distance-to-item %d", distance);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return distance;
}

/*********************************************
/ PITCH TO ITEM WRAPPER
*********************************************/
float EXPORT BOTEXP_PitchToItem(int itemType, int index)
{
	int pitch = pThinkingBot->pitch_to_item(itemType, index);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "pitch-to-item %d", pitch);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return pitch;
}

/*********************************************
/ CAN GET ITEM WRAPPER
*********************************************/
bool EXPORT BOTEXP_CanGetItem(int itemType, int index)
{
	bool can_get = pThinkingBot->can_get_item(itemType, index);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		if (can_get)
			sprintf(debugStr, "can-get-item TRUE");
		else
			sprintf(debugStr, "can-get-item FALSE");
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return can_get;
}

/*********************************************
/ GET X WRAPPER
*********************************************/
int EXPORT BOTEXP_GetX()
{
	int xpos = pThinkingBot->get_x();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "get-x %d", xpos);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return xpos;
}
/*********************************************
/ GET Y WRAPPER
*********************************************/	
int EXPORT BOTEXP_GetY()
{
	int ypos = pThinkingBot->get_y();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "get-y %d", ypos);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return ypos;
}
/*********************************************
/ GET Z WRAPPER
*********************************************/
int EXPORT BOTEXP_GetZ()
{
	int zpos = pThinkingBot->get_z();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "get-z %d", zpos);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return zpos;
}

/*********************************************
/ GET CURRENT WEAPON WRAPPER
*********************************************/
int EXPORT BOTEXP_GetCurrentWeapon()
{
	int currentWeapon = pThinkingBot->get_current_weapon();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "current-weapon %d", currentWeapon);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return currentWeapon;
}

/*********************************************
/ IS FRIEND WRAPPER
*********************************************/
bool EXPORT BOTEXP_IsFriend(int index)
{
	bool is_friend = pThinkingBot->is_friend(index);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		if (is_friend)
			sprintf(debugStr, "is_friend TRUE");
		else
			sprintf(debugStr, "is_friend FALSE");
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return is_friend;
}

/*********************************************
/ PITCH TO WAYPOINT WRAPPER
*********************************************/
float EXPORT BOTEXP_PitchToWaypoint(int index)
{
	int pitch = pThinkingBot->pitch_to_waypoint(index);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "pitch-to-waypoint %d", pitch);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return pitch;
}

/*********************************************
/ PITCH TO INFOPOINT WRAPPER
*********************************************/
float EXPORT BOTEXP_PitchToInfopoint(int infopointType, int index)
{
	int pitch = pThinkingBot->pitch_to_infopoint(infopointType, index);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "pitch-to-infopoint %d", pitch);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return pitch;
}

/*********************************************
/ GET PRIMARY AMMO IN STOCK
*********************************************/
int EXPORT BOTEXP_PrimaryAmmoInStock(int gunID)
{
	int ammo = pThinkingBot->primary_ammo_in_stock(gunID);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "ammo-in-stock-%d %d", gunID, ammo);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return ammo;
}
/*********************************************
/ GET SECONDARY AMMO IN STOCK
*********************************************/
int EXPORT BOTEXP_SecondaryAmmoInStock(int gunID)
{
	int ammo = pThinkingBot->secondary_ammo_in_stock(gunID);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		sprintf(debugStr, "ammo-in-stock-%d %d", gunID, ammo);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return ammo;
}
/*********************************************
/ GET POSSESSES GUN
*********************************************/
bool EXPORT BOTEXP_GetPossessesGun(int gunID)
{
	bool have_gun = pThinkingBot->get_possesses_gun(gunID);
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		if (have_gun)
			sprintf(debugStr, "have_gun-%d TRUE", gunID);
		else
			sprintf(debugStr, "have_gun-%d FALSE", gunID);
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return have_gun;
}

/*********************************************
/ ON LADDER WRAPPER
*********************************************/
bool EXPORT BOTEXP_OnLadder()
{
	bool onLadder = pThinkingBot->IsOnLadder();
	if (pThinkingBot->m_bDebugOn && pThinkingBot->m_bAutoSensorDebug)
	{
		if (onLadder)
			sprintf(debugStr, "is-on-ladder TRUE");
		else
			sprintf(debugStr, "is-on-ladder FALSE");
		SendExternalDebug(pThinkingBot->playerNum, debugStr);
	}
	return onLadder;
}

/*********************************************
/ ADD WAYPOINT WRAPPER
*********************************************/
void EXPORT BOTEXP_AddWaypoint()
{
	pThinkingBot->add_waypoint();
}

/*********************************************
/ ADD INFOPOINT WRAPPER
*********************************************/
void EXPORT BOTEXP_AddInfopoint(int type, int lifespan)
{
	pThinkingBot->add_infopoint(type,lifespan);
}

/*********************************************
/ STATISTIC KILLS WRAPPER
*********************************************/
int EXPORT BOTEXP_StatKills()
{
	return pThinkingBot->get_stat_kills();
}

/*********************************************
/ STATISTIC FRIENDLY KILLS WRAPPER
*********************************************/
int EXPORT BOTEXP_StatFriendlyKills()
{
	return pThinkingBot->get_stat_friendly_kills();
}

/*********************************************
/ STATISTIC DEATHS WRAPPER
*********************************************/
int EXPORT BOTEXP_StatDeaths()
{
	return pThinkingBot->get_stat_deaths();
}

/*********************************************
/ STATISTIC SUICIDES WRAPPER
*********************************************/
int EXPORT BOTEXP_StatSuicides()
{
	return pThinkingBot->get_stat_suicides();
}

/*********************************************
/ STATISTIC PERCENT TIME MOVING WRAPPER
*********************************************/
float EXPORT BOTEXP_StatPercentTimeMoving()
{
	return pThinkingBot->get_stat_percent_time_moving();
}

/********************************************
/ PLAYER NUMBER WRAPPER
********************************************/
int EXPORT BOTEXP_PlayerNum()
{
	return pThinkingBot->player_num();
}

/********************************************
/ COMMUNICATION GET INTEGER WRAPPER
********************************************/
int EXPORT BOTEXP_CommsGetInt(int player_id, int box)
{
	return pThinkingBot->comms_get_int(player_id,box);	
}

/********************************************
/ COMMUNICATION GET FLOAT WRAPPER
********************************************/
float EXPORT BOTEXP_CommsGetFloat(int player_id, int box)
{
	return pThinkingBot->comms_get_float(player_id, box);
}

/********************************************
/ COMMUNICATION GET BOOLEAN WRAPPER
********************************************/
bool EXPORT BOTEXP_CommsGetBool(int player_id, int box)
{
	return pThinkingBot->comms_get_bool(player_id, box);
}


/********************************************
/ COMMUNICATION SET INTEGER WRAPPER
********************************************/
void EXPORT BOTEXP_CommsSetInt(int box, int number)
{
	pThinkingBot->comms_set_int(box,number);
}

/********************************************
/ COMMUNICATION SET FLOAT WRAPPER
********************************************/
void EXPORT BOTEXP_CommsSetFloat(int box, float number)
{
	pThinkingBot->comms_set_float(box,number);
}

/********************************************
/ COMMUNICATION SET BOOLEAN WRAPPER
********************************************/
void EXPORT BOTEXP_CommsSetBool(int box, bool number)
{
	pThinkingBot->comms_set_bool(box,number);
}
