#include <string.h>
#include <stdio.h>
#include <time.h>
#include <fstream.h>

#include "BotTournament.h"

#include "extdll.h" 
#include "util.h"
#include "cbase.h"
#include "player.h"
#include "gamerules.h"
#include "bot.h"


// #include "TacticalAnalysis.h"

extern CGameRules * g_pGameRules;

// Socket for a stats server
// SocketClient statSocket(5441);
// SocketClient posSocket(5442);

// Command listening socket  ****** NOW DEPRECATED -- TAKE THIS OUT WHEN PATTON COMMAND CODE IS TAKEN OUT *********
// ----> Can we declare this elsewhere? It makes no sense here (but it works)
// SocketServer commandSocket(5440);

BotTacticalStats::BotTacticalStats()
{
	for (int i = 0; i < NUM_TACTICAL_HEIGHTS; i++)
	{
		for (int j = 0; j < NUM_TACTICAL_DISTANCES; j++)
			killPositions[i][j] = 0;
	}
}

void BotTournament::UpdateTacticalKills(const char * teamName, _TacticalHeightType height, _TacticalDistanceType distance)
{
	// Bail out if we're not in teamplay mode
	if (!g_pGameRules->IsTeamplay())
		return;

	// Find the appropriate team
	BotTourneyTeam * updateTeam = NULL;
	if (strcmp(teamName, team1.teamName) == 0)
		updateTeam = &team1;
	else if (strcmp(teamName, team2.teamName) == 0)
		updateTeam = &team2;
	else
		return;

	updateTeam->tacticalStats.killPositions[height][distance]++;
}

BotWeaponCounter::BotWeaponCounter(const char * newName)
{
	strcpy(weaponName, newName);
	kills = 0;
	suicides = 0;
	friendlyKills = 0;
}

BotWeaponStats::BotWeaponStats()
{
	numWeapons = 0;
}

void BotWeaponStats::UpdateWeaponCount(const char * weaponName, int killType)
{
	// Find the appropriate weapon
	bool weaponFound = FALSE;
	for (int i = 0; i < numWeapons; i++)
	{
		if (strcmp(weapons[i]->weaponName, weaponName) == 0)
		{
			weaponFound = TRUE;
			break;
		}
	}

	int weaponIndex = i;

	// If the weapon was not found, add it to the array
	if (!weaponFound)
	{
		// Bail out if we're going to overflow the array
		if (numWeapons == MAX_WEAPONS_FOR_STATS)
			return;

		weapons[numWeapons] = new BotWeaponCounter(weaponName);
		weaponIndex = numWeapons;
		numWeapons++;
	}

	// Update the weapon's count
	switch (killType)
	{
	case SCORE_KILL:
		weapons[weaponIndex]->kills++;
		//printf("[scored a kill with a %s]\n", weaponName);
		break;
	case SCORE_SUICIDE:
		weapons[weaponIndex]->suicides++;
		//printf("[scored a suicide with a %s]\n", weaponName);
		break;
	case SCORE_FRIENDLY:
		weapons[weaponIndex]->friendlyKills++;
		//printf("[scored a friendlyKill with a %s]\n", weaponName);
		break;
	default:
		printf("SCORING ERROR: Unable to tally kill of type %d\n", killType);
	}
}

void BotTournament::LoadSettings()
{
	ifstream tourneyFile;
	char filename[1000];
	BuildModFilename(filename, "tournament.txt");
	tourneyFile.open( filename );

	if ( !tourneyFile.is_open() ) {
		printf("Could not open tournament.txt\n");
		BotDebug("Could not open tournament.txt\n");
	}
	else
	{
		// Read the tournament results file destination
		prefsLoaded = TRUE;
		if (!tourneyFile.eof())
			tourneyFile.getline(prefFileDest, 1000);
		else
			prefsLoaded = FALSE;

		if (!tourneyFile.eof())
			tourneyFile.getline(tourneyName, 100);
		else
			prefsLoaded = FALSE;
	
		tourneyFile.close();
	}
	char msg[150];
	sprintf(msg, "Loaded tournament: %s\n", tourneyName);
	printf(msg);
	BotDebug(msg);
}

BotTournament::BotTournament(const char * team1Name, const char * team1Behavior, const char * team2Name, const char * team2Behavior)
{
	strcpy(team1.teamName, team1Name);
	strcpy(team1.behaviorName, team1Behavior);
	strcpy(team2.teamName, team2Name);
	strcpy(team2.behaviorName, team2Behavior);
	team1.score = 0;
	team1.kills = 0;
	team1.deaths = 0;
	team1.suicides = 0;
	team1.friendlyKills = 0;
	team2.score = 0;
	team2.kills = 0;
	team2.deaths = 0;
	team2.suicides = 0;
	team2.friendlyKills = 0;

	prefsLoaded = FALSE;
	restartNotify = TRUE;
	nextUpdateTime = 0;
}

void BotTournament::UpdateWeaponScore(const char * teamName, const char * weaponName, int killType)
{
	// Bail out if we're not in teamplay mode
	if (!g_pGameRules->IsTeamplay())
		return;

	// Find the appropriate team
	BotTourneyTeam * updateTeam = NULL;
	if (strcmp(teamName, team1.teamName) == 0)
		updateTeam = &team1;
	else if (strcmp(teamName, team2.teamName) == 0)
		updateTeam = &team2;
	else
	{
		printf("SCORING ERROR: Unable to find team '%s' to mark kill with weapon '%s'\n", teamName, weaponName);
		return;
	}

	// Update the weapon count for that team
	updateTeam->weaponStats.UpdateWeaponCount(weaponName, killType);
}

void BotTournament::UpdateTeamScore(const char * teamName, int scoreChangeType, int scoreChange)
{
	printf("In UpdateTeamScore()\n");
	
	// Bail out if we're not in teamplay mode
	if (!g_pGameRules->IsTeamplay())
		return;

	

	// Find the appropriate team
	BotTourneyTeam * updateTeam = NULL;
	if (strcmp(teamName, team1.teamName) == 0)
		updateTeam = &team1;
	else if (strcmp(teamName, team2.teamName) == 0)
		updateTeam = &team2;
	else
		return;

	char statUpdateMsg[500];

	// Update the team score and stats appropriately
	switch (scoreChangeType)
	{
	case SCORE_KILL:
		updateTeam->kills += scoreChange;

		// Send an update to the stats server
		sprintf(statUpdateMsg, "stat team %s kills %d\n", teamName, updateTeam->kills);
		BotDebug(statUpdateMsg);
//		statSocket.SendMessage(statUpdateMsg);
		printf("Send a kill message to the socket (maybe?)\n");
		
	
		updateTeam->score += scoreChange;
		break;
	case SCORE_SUICIDE:
		updateTeam->suicides -= scoreChange;
		updateTeam->score += scoreChange;
		break;
	case SCORE_FRIENDLY:
		updateTeam->friendlyKills -= scoreChange;
		updateTeam->score += scoreChange;
		break;
	case SCORE_DEATH:
		updateTeam->deaths += scoreChange;
		break;
	}
}

void BotTournament::DumpScoreReport()
{	
	if (!prefsLoaded)
	{
		BotDebug("Loading tournament settings...\n");
		LoadSettings();
	}

	if (prefsLoaded)
	{
		char filename[1112];
		sprintf(filename, "%s\\bot_tourney_%s.txt", prefFileDest, tourneyName);
		FILE * dumpFile;
		dumpFile = fopen(filename, "w");
		if (!dumpFile)
		{
			BotDebug("Can't open the tournament preferences file.\n");
		}
		else
		{
			// If this is a new tournament, print a "restart" notification
			if (restartNotify)
			{
				fprintf(dumpFile, "restart\n\n");
				restartNotify = FALSE;
			}

			// Print out team 1 scores
			fprintf(dumpFile, "team1name = %s\n", team1.teamName);
			fprintf(dumpFile, "team1dll = %s\n", team1.behaviorName);
			fprintf(dumpFile, "team1score = %d\n", team1.score);
			fprintf(dumpFile, "team1kills = %d\n", team1.kills);
			fprintf(dumpFile, "team1deaths = %d\n", team1.deaths);
			fprintf(dumpFile, "team1suicides = %d\n", team1.suicides);
			fprintf(dumpFile, "team1friendly_kills = %d\n", team1.friendlyKills);

			// Print out team 1 weapon report
			for (int j = 0; j < team1.weaponStats.numWeapons; j++)
			{
				BotWeaponCounter * weapon = team1.weaponStats.weapons[j];
				fprintf(dumpFile, "team1weapon_%s_kills = %d\n", weapon->weaponName, weapon->kills);
				fprintf(dumpFile, "team1weapon_%s_suicides = %d\n", weapon->weaponName, weapon->suicides);
				fprintf(dumpFile, "team1weapon_%s_friendly = %d\n", weapon->weaponName, weapon->friendlyKills);
			}

			// Print out the tactical analysis report
			fprintf(dumpFile, "team1tactical_above_close_kill = %d\n", team1.tacticalStats.killPositions[Above][Close]);
			fprintf(dumpFile, "team1tactical_above_medium_kill = %d\n", team1.tacticalStats.killPositions[Above][Medium]);
			fprintf(dumpFile, "team1tactical_above_far_kill = %d\n", team1.tacticalStats.killPositions[Above][Far]);
			fprintf(dumpFile, "team1tactical_level_close_kill = %d\n", team1.tacticalStats.killPositions[Level][Close]);
			fprintf(dumpFile, "team1tactical_level_medium_kill = %d\n", team1.tacticalStats.killPositions[Level][Medium]);
			fprintf(dumpFile, "team1tactical_level_far_kill = %d\n", team1.tacticalStats.killPositions[Level][Far]);
			fprintf(dumpFile, "team1tactical_below_close_kill = %d\n", team1.tacticalStats.killPositions[Below][Close]);
			fprintf(dumpFile, "team1tactical_below_medium_kill = %d\n", team1.tacticalStats.killPositions[Below][Medium]);
			fprintf(dumpFile, "team1tactical_below_far_kill = %d\n", team1.tacticalStats.killPositions[Below][Far]);
			
			// Print a seperator
			fprintf(dumpFile, "\n");
			
			// Print out team 2 scores
			fprintf(dumpFile, "team2name = %s\n", team2.teamName);
			fprintf(dumpFile, "team2dll = %s\n", team2.behaviorName);
			fprintf(dumpFile, "team2score = %d\n", team2.score);
			fprintf(dumpFile, "team2kills = %d\n", team2.kills);
			fprintf(dumpFile, "team2deaths = %d\n", team2.deaths);
			fprintf(dumpFile, "team2suicides = %d\n", team2.suicides);
			fprintf(dumpFile, "team2friendly_kills = %d\n", team2.friendlyKills);

			// Print out team 2 weapon report
			for (int k = 0; k < team2.weaponStats.numWeapons; k++)
			{
				BotWeaponCounter * weapon = team2.weaponStats.weapons[k];
				fprintf(dumpFile, "team2weapon_%s_kills = %d\n", weapon->weaponName, weapon->kills);
				fprintf(dumpFile, "team2weapon_%s_suicides = %d\n", weapon->weaponName, weapon->suicides);
				fprintf(dumpFile, "team2weapon_%s_friendly = %d\n", weapon->weaponName, weapon->friendlyKills);
			}

			// Print out the tactical analysis report
			fprintf(dumpFile, "team2tactical_above_close_kill = %d\n", team2.tacticalStats.killPositions[Above][Close]);
			fprintf(dumpFile, "team2tactical_above_medium_kill = %d\n", team2.tacticalStats.killPositions[Above][Medium]);
			fprintf(dumpFile, "team2tactical_above_far_kill = %d\n", team2.tacticalStats.killPositions[Above][Far]);
			fprintf(dumpFile, "team2tactical_level_close_kill = %d\n", team2.tacticalStats.killPositions[Level][Close]);
			fprintf(dumpFile, "team2tactical_level_medium_kill = %d\n", team2.tacticalStats.killPositions[Level][Medium]);
			fprintf(dumpFile, "team2tactical_level_far_kill = %d\n", team2.tacticalStats.killPositions[Level][Far]);
			fprintf(dumpFile, "team2tactical_below_close_kill = %d\n", team2.tacticalStats.killPositions[Below][Close]);
			fprintf(dumpFile, "team2tactical_below_medium_kill = %d\n", team2.tacticalStats.killPositions[Below][Medium]);
			fprintf(dumpFile, "team2tactical_below_far_kill = %d\n", team2.tacticalStats.killPositions[Below][Far]);
			
			// Put a timestamp on
			time_t now;
			now = time((time_t *)NULL);
			fprintf(dumpFile, "\ntimestamp = %s", ctime(&now));
			
			// Close the file
			fclose(dumpFile);
		}
		BotDebug("Dumped tournament info to file.\n");
	}
}

void BotTournament::WriteHtmlScoreReport(const char * filename)
{
	FILE * dumpFile;
	dumpFile = fopen(filename, "w");
	if (!dumpFile)
	{
		printf("Can't open the file %s", filename);
	}
	else
	{
		//write the html header
		fprintf(dumpFile, "<html>\n");
		fprintf(dumpFile, "<head>\n");
		fprintf(dumpFile, "<title>Bot Tournament</title>\n");
		fprintf(dumpFile, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n");
		fprintf(dumpFile, "</head>\n <body  bgcolor=\"#000000\" align= \"center\">\n<br><br>");

		fprintf(dumpFile, "<font color=\"#d6061b\" size=\"+2\"> %s(%s) vs. %s(%s) </font><br><br>" , team1.teamName, team1.behaviorName, team2.teamName , team2.behaviorName);
		// Print out team 1 scores
		if(team1.kills > team2.kills){
			fprintf(dumpFile, "<font color=\"#CCFF00\" size=\"+1\">TEAM 1: %s (%s)\n<br><br></font><font color=\"#CCFF00\">", team1.teamName, team1.behaviorName);
		}else{
			fprintf(dumpFile, "<font color=\"#0000FF\" size=\"+1\">TEAM 1: %s (%s)\n<br><br></font><font color=\"#0000FF\">", team1.teamName, team1.behaviorName);
		}
		fprintf(dumpFile, "SCORE: %d\n<br>", team1.score);
		fprintf(dumpFile, "\tKILLS: %d\n<br>", team1.kills);
		fprintf(dumpFile, "\tDEATHS: %d\n<br>", team1.deaths);
		fprintf(dumpFile, "\tSUICIDES: %d\n<br>", team1.suicides);
		fprintf(dumpFile, "\tFRIENDLY KILLS: %d\n <br></font>", team1.friendlyKills);
		
		// Print a seperator
		fprintf(dumpFile, "\n<br><br><br>");
		
		// Print out team 2 scores
		if(team1.kills > team2.kills){
			fprintf(dumpFile, "<font color=\"#0000FF\" size=\"+1\">TEAM 2: %s (%s)\n<br><br></font><font color=\"#0000FF\">", team2.teamName, team2.behaviorName);
		}else{
			fprintf(dumpFile, "<font color=\"#CCFF00\" size=\"+1\">TEAM 2: %s (%s)\n<br><br></font><font color=\"#CCFF00\">", team2.teamName, team2.behaviorName);
		}
		fprintf(dumpFile, "SCORE: %d\n<br>", team2.score);
		fprintf(dumpFile, "\tKILLS: %d\n<br>", team2.kills);
		fprintf(dumpFile, "\tDEATHS: %d\n<br>", team2.deaths);
		fprintf(dumpFile, "\tSUICIDES: %d\n<br>", team2.suicides);
		fprintf(dumpFile, "\tFRIENDLY KILLS: %d\n<br><br><br>", team2.friendlyKills);

		// Put a timestamp on
		time_t now;
		now = time((time_t *)NULL);
		fprintf(dumpFile, "\nLast updated: %s<br>", ctime(&now));

		//close the html
		fprintf(dumpFile, "</body>\n");
		fprintf(dumpFile, "</html>");
		// Close the file
		fclose(dumpFile);
	}
	BotDebug("Dumped tournament info to file.\n");
	printf("Dumped tournament info to file.\n");
}




BotPosStream::BotPosStream()
{
	running = FALSE;
	accesses = 0;
	numBots = 0;
//	prefsLoaded = FALSE;
	//ConnectServer();
	LoadSettings();
	for(int i=0;i<32;i++)
	{
		BotPoses[i][0]=-1;
		BotPoses[i][1]=-1;
		BotPoses[i][2]=-1;
		BotPoses[i][3]=-1;
	}
}

//read the server and port of the server to connect to
//from the file positionServer.txt in the mod's dir
void BotPosStream::LoadSettings()
{
	//prefServer = "127.0.0.1";
	//perfPort = "4445";
	/*
	ifstream serverConfFile;
	char filename[1000];
	BuildModFilename(filename, "positionServer.txt");
	serverConfFile.open( filename );

	if ( !serverConfFile.is_open() ) {
		printf("Could not open positionServer.txt\n");
		BotDebug("Could not open positionServer.txt\n");
	}
	else
	{
		// Read the tournament results file destination
		prefsLoaded = TRUE;
		if (!serverConfFile.eof())
			serverConfFile.getline(prefServer, 100);
		else
			prefsLoaded = FALSE;

		if (!serverConfFile.eof())
			serverConfFile.getline(prefPort, 100);
		else
			prefsLoaded = FALSE;
	
		serverConfFile.close();
	}
	char msg[150];
	sprintf(msg, "Loaded Position Server Settings: %s\n", prefsLoaded);
	printf(msg);
	BotDebug(msg);
*/
}

/*
void BotPosStream::ConnectServer(SocketClient *socket){
	socket = new SocketClient(atoi(prefPort));
}
*/

void BotPosStream::writePos(SocketClient socket){
	char posMsg[1000];
	//char tempMsg[256];
	for(int i=1;i<=numBots;i++)
	{
		if(BotPoses[i][0] != -1 && BotPoses[i][1] !=-1 && BotPoses[i][2] != -1 && BotPoses[i][2]!= -1){
			sprintf(posMsg, "<bot num=%d x=%d y=%d z=%d life=%d />\n\000", i, BotPoses[i][0] ,BotPoses[i][1],
				BotPoses[i][2],BotPoses[i][3]);
			//socket.SendMessage(posMsg);
		}
	}
		//socket->SendMessage(posMsg);

	ofstream POS_OUT;
	//output file for the position of the bots this is bad 
	POS_OUT.open("C:\\pos.txt", ofstream::out ); 
	for(i=1;i<=numBots;i++)
	{
		if(i==1)
		{
			POS_OUT	<<"bot"<<i<<"_x="<<BotPoses[i][0];
		}else{
			POS_OUT	<<"&bot"<<i<<"_x="<<BotPoses[i][0];
		}
		
		POS_OUT	<<"&bot"<<i<<"_y="<<BotPoses[i][1]
				<<"&bot"<<i<<"_facing="<<BotPoses[i][2]
				<<"&bot"<<i<<"_life="<<BotPoses[i][3];
	}
	POS_OUT.close();


	numBots =0;
	accesses=0;
}


void BotPosStream::recordPos(int botNum, int botX,int botY,int botFacing, int life){
	BotPoses[botNum][0] = botX;
	BotPoses[botNum][1] = botY;
	BotPoses[botNum][2] = botFacing;
	BotPoses[botNum][3] = life;
	accesses++;
	if(botNum > numBots)
		numBots = botNum;
}


/*
BotPosStream::BotPosStream()
{
	accesses = 0;
	numBots = 0;
	
}

void BotPosStream::writePos(){
/*	ofstream POS_OUT;
	POS_OUT.open("C:\\Documents and Settings\\ntrienens\\My Documents\\pos.txt", ofstream::out );
	for(int i=1;i<numBots;i++)
	{
		if(i==0)
		{
			POS_OUT	<<"bot"<<i<<"_x="<<BotPoses[i][0];
		}else{
			POS_OUT	<<"&bot"<<i<<"_x="<<BotPoses[i][0];
		}
		
		POS_OUT	<<"&bot"<<i<<"_y="<<BotPoses[i][1]
				<<"&bot"<<i<<"_z="<<BotPoses[i][2]
				<<"&bot"<<i<<"_life="<<BotPoses[i][3];
	}
	POS_OUT.close();
	numBots =0;
	accesses=0;
}
*/