/*********************************************
/ BotSensors.cpp
/
/ This file contains utility code relating to the
/ bots.
*********************************************/

#include "extdll.h" 
#include "util.h"
#include "cbase.h"
#include "player.h"
#include "bot.h"

#include "BotUtility.h"
#include "BotMessenger.h"
#include "sys/timeb.h"

extern CBotMessenger botMessenger;

/*********************************************
/ ROUND DECIMAL
/ Round a float to the number of digits
/ specified. Returns a float with precision to
/ that number of decimals.
*********************************************/
float roundDecimal(float number, int digitsAfterDecimal)
{
	int chopFactor = 0;
	if (digitsAfterDecimal > 0)
	{
		chopFactor = 10;
		for (int i = 1; i < digitsAfterDecimal; i++)
			chopFactor = chopFactor * 10;
	}
	else
		return floor(number);

	float roundedNum = (number - floor(number)) * chopFactor;
	roundedNum = (floor(roundedNum) / chopFactor) + floor(number);
	return roundedNum;
}

/*********************************************
/ BOT-UTIL ROTATE ANGLE
/ Rotates and angle by the given offset and returns
/ the resulting angle (after adjusting for wraps).
/ The rotation angle is taken in degrees.
*********************************************/
float BotUTIL_RotateAngle(float angle, float offset)
{
	// Do the rotation
	angle += offset;

	// Check for wrap around of angle
	if (angle < 0)
		angle += 360;
	if (angle >= 360)
		angle -= 360;
	
	return angle;
}

/*********************************************
/ BOT-UTIL ROTATE VECTOR Z
/ Rotates the given vector around the z-axis by the
/ specified rotation angle. The rotation angle is
/ taken in degrees.
*********************************************/
Vector BotUTIL_RotateVectorZ(Vector vec, float rotation)
{
	float angle = degToRad(rotation);
	return Vector (vec.x * cos (angle) - vec.y * sin(angle),
				  vec.x * sin (angle) + vec.y * cos(angle),
				  vec.z);
}

/*********************************************
/ BOT-UTIL ROTATE VECTOR Y
/ Rotates the given vector around the y-axis by the
/ specified rotation angle. The rotation angle is
/ taken in degrees.
*********************************************/
Vector BotUTIL_RotateVectorY(Vector vec, float rotation)
{
	float angle = degToRad(rotation);
	return Vector(vec.x * cos(angle) + vec.z * sin(angle),
					vec.y,
					vec.z * cos(angle) - vec.x * sin(angle));
}

/*********************************************
/ DEGREES TO RADIANS
/ Converts degrees to radians.
*********************************************/
float degToRad(float angle)
{
	return (float) (angle * (PI/180));
}

/*********************************************
/ RADIANS TO DEGREES
/ Converts radians to degrees.
*********************************************/
float radToDeg(float angle)
{
	return (float) (angle * (180/PI));
}

/*********************************************
/ TOGGLE
/ Toggles a boolean flag (returns the opposite).
*********************************************/
bool toggle(bool flag)
{
	return (!flag);
}

/*********************************************
/ PAUSE ALL BOTS
/ Pauses all bots in the game
*********************************************/
void PauseAllBots()
{
	// Send a "pause" message to each player that is a bot
	for (int i = 1; i <= gpGlobals->maxClients; i++ )
	{
		CBasePlayer *pPlayer = (CBasePlayer *)UTIL_PlayerByIndex( i );
		if(pPlayer && pPlayer->isBot)
		{
			CBotMessage * pauseMsg = new CBotMessage(-1, i, "pause", "on");
			botMessenger.PostBotMessage(pauseMsg);
		}
	}
}

/*********************************************
/ UNPAUSE ALL BOTS
/ Unpauses all bots in the game
*********************************************/
void UnpauseAllBots()
{
	// Send a "pause" message to each player that is a bot
	for (int i = 1; i <= gpGlobals->maxClients; i++ )
	{
		CBasePlayer *pPlayer = (CBasePlayer *)UTIL_PlayerByIndex( i );
		if(pPlayer && pPlayer->isBot)
		{
			CBotMessage * pauseMsg = new CBotMessage(-1, i, "pause", "off");
			botMessenger.PostBotMessage(pauseMsg);
		}
	}
}

/*********************************************
/ DEBUG ALL BOTS ON
/ Toggles the debug status of all of the bots
*********************************************/
void DebugAllBotsOn()
{
	// Send a "debug on" message to each player that is a bot
	for (int i = 1; i <= gpGlobals->maxClients; i++ )
	{
		CBasePlayer *pPlayer = (CBasePlayer *)UTIL_PlayerByIndex( i );
		if(pPlayer && pPlayer->isBot)
		{
			CBotMessage * loadDllMsg = new CBotMessage(-1, i, "debug", "on");
			botMessenger.PostBotMessage(loadDllMsg);
		}
	}
}

/*********************************************
/ DEBUG ALL BOTS OFF
/ Toggles the debug status of all of the bots
*********************************************/
void DebugAllBotsOff()
{
	// Send a "debug off" message to each player that is a bot
	for (int i = 1; i <= gpGlobals->maxClients; i++ )
	{
		CBasePlayer *pPlayer = (CBasePlayer *)UTIL_PlayerByIndex( i );
		if(pPlayer && pPlayer->isBot)
		{
			CBotMessage * loadDllMsg = new CBotMessage(-1, i, "debug", "off");
			botMessenger.PostBotMessage(loadDllMsg);
		}
	}
}

/*********************************************
/ GET SYSTEM TIME IN MILLISECONDS
/ Gets the current system time in milliseconds.
*********************************************/
long GetSysTimeMillisec()
{
	struct timeb my_time;
	ftime(&my_time);
	long systime = (my_time.time * 1000) + my_time.millitm;
	return systime;
}

/*********************************************
/ PRINT ABOUT FLEX-BOT
/ Prints info about the FlexBot
*********************************************/
void PrintAboutFlexBot()
{
	BotDebug("-------------------------\n");
	BotDebug("FlexBot (Version Alpha 2)\n");
	BotDebug("Northwestern University\n");
	BotDebug("-------------------------\n");
}

/*********************************************
/ FLEX-BOT HELP
/ Displays help system for the FlexBot.
*********************************************/
void FlexBotHelp()
{
	BotDebug("---------------\n");
	BotDebug("FlexBot Help\n");
	BotDebug("---------------\n");
	BotDebug("Available FlexBot console commands:\n");
	BotDebug("  bot add\n");
	BotDebug("  bot add [n]\n");
	BotDebug("  bot remove [player-number]\n");
	BotDebug("  bot menu\n");
	BotDebug("  bot pause [on/off]\n");
	BotDebug("  bot debug [on/off]\n");
	BotDebug("  bot freecam [on/off]\n");
	BotDebug("  bot chasecam [player-number] [on/off]\n");
	BotDebug("  bot about\n");
	BotDebug("  bot help\n");
	BotDebug(" For individual bots:\n");
	BotDebug("   bot [n] dll [filename]\n");
	BotDebug("   bot [n] pause [on/off]\n");
	BotDebug("   bot [n] debug [on/off]\n");
	BotDebug("   bot [n] autodebug [on/off]\n");
	BotDebug("   bot [n] waypoint add\n");
	BotDebug("   bot [n] waypoint remove\n");
	BotDebug("   bot [n] waypoint load [filename]\n");
	BotDebug("   bot [n] waypoint export [filename]\n");
	BotDebug("   bot [n] infopoint add [type]\n");
	BotDebug("   bot [n] infopoint remove\n");
	BotDebug("   bot [n] skill [skill-level: 0-100]\n");
	BotDebug("  For all bots:\n");
	BotDebug("   bot all [any bot command]\n");
	BotDebug("   bot tournament [behavior 1] [behavior 2]\n");
	BotDebug("  For groups of bots:\n");
	BotDebug("   bot [bot 1] [bot 2] [bot n] [any bot command]\n"); 
	BotDebug("  For teams of bots:\n");
	BotDebug("   bot team [teamname] [any bot command]\n");
}

/*********************************************
/ BUILD MOD FILENAME
/ Returns a path to the given filename within the
/ MOD directory. For example, if you want a path to
/ the file "foo.txt" in the MOD directory, you would
/ call BuildModFilename("foo.txt") and be given a
/ valid path.
*********************************************/
void BuildModFilename(char * path, const char * filename)
{
	sprintf(path, "%s%s", "FlexBot/", filename);
}
/*********************************************
/ IS BOT NUMBER
/ check to seee if a char * is actually a number that could be a bot number
**********************************************/
bool isBotNumber(const char * arg)
{
	if( !strcmp(arg,"1")   ||!strcmp(arg,"2")  ||!strcmp(arg,"3") ||!strcmp(arg,"4")
 		||!strcmp(arg,"5") ||!strcmp(arg,"6")  ||!strcmp(arg,"7")  ||!strcmp(arg,"8") 
		||!strcmp(arg,"9")  ||!strcmp(arg,"10") ||!strcmp(arg,"11") ||!strcmp(arg,"12")
		||!strcmp(arg,"13") ||!strcmp(arg,"14") ||!strcmp(arg,"15") ||!strcmp(arg,"16")
		||!strcmp(arg,"17") ||!strcmp(arg,"18") ||!strcmp(arg,"19") ||!strcmp(arg,"20")
		||!strcmp(arg,"21") ||!strcmp(arg,"23") ||!strcmp(arg,"24") ||!strcmp(arg,"25")
		||!strcmp(arg,"26") ||!strcmp(arg,"27") ||!strcmp(arg,"28") ||!strcmp(arg,"29")
		||!strcmp(arg,"30") ||!strcmp(arg,"31") ||!strcmp(arg,"32"))
	{
		return TRUE;
	}	
	return FALSE;
}