/*********************************************
/ ChaseCam.cpp
/
/ This file contains code to run the chase
/ camera, which follows any player on which
/ it is fixed. Based on Botman's botcam.
*********************************************/

#include "extdll.h"
#include "util.h"
#include "client.h"
#include "cbase.h"
#include "player.h"

#include "BotUtility.h"
#include "ChaseCam.h"
#include "bot.h"


LINK_ENTITY_TO_CLASS(entity_chasecam, CChaseCam);


CChaseCam *CChaseCam::Create( CBasePlayer *pPlayer, CBasePlayer *pTarget )
{
	// Disconnect the freecam if the player is already in that mode
	if(pPlayer->pFreeCam)
		pPlayer->pFreeCam->Disconnect();

	CChaseCam * pChaseCam = GetClassPtr( (CChaseCam *)NULL );

	PRECACHE_MODEL( "models/mechgibs.mdl" );

	pChaseCam->m_pPlayer = pPlayer;
	pChaseCam->m_pTarget = pTarget;

	pPlayer->pev->effects |= EF_NODRAW;
	pPlayer->pev->solid = SOLID_NOT;
	pPlayer->pev->takedamage = DAMAGE_NO;
	// pPlayer->pev->movetype = MOVETYPE_NONE;
	// pPlayer->m_iHideHUD |= HIDEHUD_ALL;

	// Hide everything except for chat messages and menus
	pPlayer->m_iHideHUD |= HIDEHUD_HEALTH;
	pPlayer->m_iHideHUD |= HIDEHUD_WEAPONS;

	pChaseCam->Spawn();

	return pChaseCam;
}
   

void CChaseCam::Spawn( void )
{
	if (!m_pPlayer || !m_pTarget)
		return;

	pev->classname = MAKE_STRING("entity_chasecam");

	UTIL_MakeVectors(m_pTarget->pev->v_angle);

	// Construct a vector from the origin of following length
	Vector cameraView (CHASECAM_DIST, 0, 0);

	// Rotate the vision vector by checkAngle degrees from the player's view
	float rotateAngle = m_pTarget->pev->v_angle.y + m_pPlayer->pev->v_angle.y;
	cameraView = BotUTIL_RotateVectorZ(cameraView, rotateAngle);

	// Trace so that the camera does not appear behind an obstacle
	TraceResult tr;
	UTIL_TraceLine(m_pTarget->pev->origin + m_pTarget->pev->view_ofs,
					m_pTarget->pev->origin + m_pTarget->pev->view_ofs + cameraView + Vector(0, 0, 25),
					dont_ignore_monsters, m_pTarget->edict(), &tr );

	// Set the camera's position
	UTIL_SetOrigin(pev, tr.vecEndPos);

	// Set the camera's view angle
	pev->angles.y = UTIL_VecToAngles((m_pTarget->pev->origin + m_pTarget->pev->view_ofs) - tr.vecEndPos).y;
	pev->fixangle = TRUE;

	SET_VIEW (m_pPlayer->edict(), edict());

	// Set the camera's model to be an invisible model
	SET_MODEL(ENT(pev), "models/mechgibs.mdl");
	pev->movetype = MOVETYPE_FLY;
	pev->solid = SOLID_NOT;
	pev->takedamage = DAMAGE_NO;
	pev->renderamt = 0;

	m_pPlayer->EnableControl(TRUE);

	SetTouch( NULL );

	SetThink( IdleThink );

	// Tell the player who they are looking at
	UTIL_ShowMessage(UTIL_VarArgs("Chase cam is following %s\n", STRING(m_pTarget->pev->netname)), m_pPlayer);

	pev->nextthink = gpGlobals->time;
}

void CChaseCam::ViewNextPlayer()
{
	int currentTarget = GetPlayerIndex(m_pTarget);
	int goodTarget = currentTarget;
	int loopBackTarget = -1;

	// Find the next target (or loop back)
	for (int newTarget = 1; newTarget <= gpGlobals->maxClients; newTarget++)
	{
		// Don't check the current target
		if (newTarget == currentTarget)
			continue;

		CBasePlayer * pNewPlayer = (CBasePlayer *)UTIL_PlayerByIndex( newTarget );

		// Skip invalid players
		if (!pNewPlayer)
			continue;

		// Don't watch players in chase cam mode
		if (pNewPlayer->pChaseCam)
			continue;

		// Skip this player if dead or dying
		if (pNewPlayer->pev->deadflag != DEAD_NO)
			continue;

		if (loopBackTarget == -1)
			loopBackTarget = newTarget;

		// Store this player as the last good one
		goodTarget = newTarget;

		// Stop looking if the client number is greater than the current target
		if (newTarget > currentTarget)
			break;
	}

	if (goodTarget < currentTarget)
		goodTarget = loopBackTarget;

	// Set the new target
	ChangeViewToPlayer( (CBasePlayer *)UTIL_PlayerByIndex( goodTarget ) );
}

void CChaseCam::ViewPreviousPlayer()
{
	int currentTarget = GetPlayerIndex(m_pTarget);
	int goodTarget = currentTarget;
	int loopBackTarget = -1;

	// Find the next target (or loop back)
	for (int newTarget = gpGlobals->maxClients; newTarget >= 1; newTarget--)
	{
		// Don't check the current target
		if (newTarget == currentTarget)
			continue;

		CBasePlayer * pNewPlayer = (CBasePlayer *)UTIL_PlayerByIndex( newTarget );

		// Skip invalid players
		if (!pNewPlayer)
			continue;

		// Don't watch players in chase cam mode
		if (pNewPlayer->pChaseCam)
			continue;

		// Skip this player if dead or dying
		if (pNewPlayer->pev->deadflag != DEAD_NO)
			continue;

		if (loopBackTarget == -1)
			loopBackTarget = newTarget;

		// Store this player as the last good one
		goodTarget = newTarget;

		// Stop looking if the client number is greater than the current target
		if (newTarget < currentTarget)
			break;
	}

	if (goodTarget > currentTarget)
		goodTarget = loopBackTarget;

	// Set the new target
	ChangeViewToPlayer( (CBasePlayer *)UTIL_PlayerByIndex( goodTarget ) );
}

void CChaseCam::ChangeViewToPlayer(CBasePlayer * pTarget)
{
	if(m_pPlayer == pTarget)	
		return;
	// Set the new target
	m_pTarget = pTarget;

	// Tell the player who they are looking at
	UTIL_ShowMessage(UTIL_VarArgs("%s [Health %d]\n", STRING(pTarget->pev->netname), pTarget->m_iClientHealth), m_pPlayer);
	//UTIL_CenterPrintAll(UTIL_VarArgs("Chase cam is following %s\n", STRING(pTarget->pev->netname)));
}

void CChaseCam::IdleThink( void )
{
	if (!m_pPlayer || !m_pTarget)
		return;

	// ***** TAKE ME OUT -- FOR TESTING PURPOSES ONLY *****
	// pev->movetype = m_pTarget->pev->movetype;
	// ***************************************************/

	// Adjust player's view angles to prevent screwy wrap-around views
	if (m_pPlayer->pev->v_angle.x > 90)
		m_pPlayer->pev->v_angle.x = 90;
	else if (m_pPlayer->pev->v_angle.x < -90)
		m_pPlayer->pev->v_angle.x = -90;
	m_pPlayer->pev->angles.x = m_pPlayer->pev->v_angle.x / 3;

	m_pPlayer->pev->v_angle.z = 0;
	m_pPlayer->pev->angles.z = m_pPlayer->pev->v_angle.z;

	// Check if the user wants to cycle to the NEXT player target
	if (!(m_pPlayer->pev->button & IN_ATTACK))
	{
		// Reactivate player switching (so that holding down the
		// button doesn't cycle rapidly)
		bCanSwitchTarget = TRUE;
	}
	else if ((m_pPlayer->pev->button & IN_ATTACK) && bCanSwitchTarget)
	{
		ViewNextPlayer();

		// Deactivate player switching until button is released
		bCanSwitchTarget = FALSE;
	}

	// Check if the user wants to cycle to the PREVIOUS target
	if (!(m_pPlayer->pev->button & IN_ATTACK2))
	{
		// Reactivate player switching (so that holding down the
		// button doesn't cycle rapidly
		bCanSwitchTargetPrev = TRUE;
	}
	else if ((m_pPlayer->pev->button & IN_ATTACK2) && bCanSwitchTargetPrev)
	{
		ViewPreviousPlayer();

		// Deactivate player switching until button is released
		bCanSwitchTargetPrev = FALSE;
	}

	// If the target is still in the game, then view it
	if (m_pTarget->pev->takedamage != DAMAGE_NO)  // not kicked
	{ 
		UTIL_MakeVectors(m_pTarget->pev->v_angle);

		// Construct a vector from the origin of following length
		////Vector cameraView (CHASECAM_DIST, 0, 0);
		Vector cameraView ( CHASECAM_DIST * cos(degToRad(m_pPlayer->pev->v_angle.x)),
							0, 
							CHASECAM_DIST*sin(degToRad(m_pPlayer->pev->v_angle.x)));

		// Adjust the camera's position based on yaw control
		float rotateAngleZ = m_pTarget->pev->v_angle.y + m_pPlayer->pev->v_angle.y;
		cameraView = BotUTIL_RotateVectorZ(cameraView, rotateAngleZ);

		// Adjust the camera's position based on pitch control
		////float rotateAngleY = 90 * (m_pPlayer->pev->v_angle.x / 90);
		////cameraView = BotUTIL_RotateVectorY(cameraView, rotateAngleY);

		// Trace so that the camera does not appear behind an obstacle
		TraceResult tr;
		UTIL_TraceLine(m_pTarget->pev->origin + m_pTarget->pev->view_ofs,
						m_pTarget->pev->origin + m_pTarget->pev->view_ofs + cameraView + Vector(0, 0, 25),
						dont_ignore_monsters, m_pTarget->edict(), &tr );

		// Set the camera's position
		UTIL_SetOrigin(pev, tr.vecEndPos);

		// Set the camera's view angle
		pev->angles.y = UTIL_VecToAngles((m_pTarget->pev->origin + m_pTarget->pev->view_ofs) - tr.vecEndPos).y;
		pev->angles.x = -UTIL_VecToAngles((m_pTarget->pev->origin + m_pTarget->pev->view_ofs) - tr.vecEndPos).x;
		pev->fixangle = TRUE;

		// Give the player the camera's view
		// SET_VIEW (m_pPlayer->edict(), edict());
		pev->nextthink = gpGlobals->time + 0.01;

		// Display the "chased" player's name and health
		UTIL_ShowMessage(UTIL_VarArgs("%s [Health %d]\n", STRING(m_pTarget->pev->netname), m_pTarget->m_iClientHealth), m_pPlayer);
	}
	else
	{
		// Get rid of the chase cam
		SET_VIEW (m_pPlayer->edict(), m_pPlayer->edict());
		m_pPlayer->pev->effects &= ~EF_NODRAW;
		m_pPlayer->pev->solid = SOLID_SLIDEBOX;
		m_pPlayer->pev->takedamage = DAMAGE_AIM;
		m_pPlayer->m_iHideHUD &= ~HIDEHUD_ALL;
		m_pPlayer->m_iHideHUD &= ~HIDEHUD_HEALTH;
		m_pPlayer->m_iHideHUD &= ~HIDEHUD_WEAPONS;
		m_pPlayer->EnableControl(TRUE);
		m_pPlayer->pChaseCam = NULL;  // player's chase cam is no longer valid
		m_pTarget = NULL;
		m_pPlayer = NULL;
		REMOVE_ENTITY( ENT(pev) );
	}
}


void CChaseCam::Disconnect( void )
{
	SET_VIEW (m_pPlayer->edict(), m_pPlayer->edict());

	m_pPlayer->pev->effects &= ~EF_NODRAW;
	m_pPlayer->pev->solid = SOLID_SLIDEBOX;
	m_pPlayer->pev->takedamage = DAMAGE_AIM;
	m_pPlayer->m_iHideHUD &= ~HIDEHUD_ALL;
	m_pPlayer->m_iHideHUD &= ~HIDEHUD_HEALTH;
	m_pPlayer->m_iHideHUD &= ~HIDEHUD_WEAPONS;

	m_pPlayer->EnableControl(TRUE);

	m_pPlayer->pChaseCam = NULL;  // player's chase cam is no longer valid


	m_pTarget = NULL;
	m_pPlayer = NULL;

	REMOVE_ENTITY( ENT(pev) );
}

