/*********************************************
/ Commands.cpp
/
/ This file contains code to process a command
/ from the player.
*********************************************/

#include "extdll.h" 
#include "util.h"
#include "client.h"
#include "cbase.h"

#include "Commands.h"
#include "stdio.h"
#include "string.h"
#include "bot.h"
#include "BotUtility.h"
#include "BotMessenger.h"
#include "BotTournament.h"
#include "ChaseCam.h"
#include "FreeCam.h"
#include "gamerules.h"

extern CBotMessenger botMessenger;
extern CGameRules * g_pGameRules;
extern BotTournament * tournament;
extern int botsWaitingToBeAdded;

void ProcessCommand(CBasePlayer * pIssuingPlayer, const char * cmd)
{
	char * argv[MAX_ARGS];
	int argc = 0;
	char * token;

	// Get a copy of the command because strtok is destructive
	char cmdcpy[MAX_CMD_LENGTH];
	strcpy(cmdcpy, cmd);

	// Parse the commands into an array
	token = strtok(cmdcpy, " ");
	while (token != NULL)
	{
		argv[argc] = token;
		++argc;
		token = strtok(NULL, " ");
	}

	// Process the commands
	if ( FStrEq(argv[0], "bot") && argc >= 2 )
	{
		if ( FStrEq(argv[1], "all") )
		{
			if (argc > 2)
			{
				// Construct the message strings
				char * bot_command = (char *)argv[2];				
				char bot_argument[500];
				
				// Stitch the rest of the string up for the argument string
				if (argc > 3)
					strcpy(bot_argument, (char *)argv[3]);
				else
					strcpy(bot_argument, "*none*");
				for (int j = 4; j < argc; j++)
				{
					strcat(bot_argument, " ");
					strcat(bot_argument, (char *)argv[j]);					
				}

				// Send the command to every bot
				for (int i = 0; i < MAX_CLIENTS; i++)
				{
					CBasePlayer * pPlayer = (CBasePlayer *)UTIL_PlayerByIndex(i);
					if (pPlayer && pPlayer->isBot)
					{
						// Send the message
						CBotMessage * msgToSend = new CBotMessage(GetPlayerIndex(pIssuingPlayer), i, bot_command, bot_argument);
						botMessenger.PostBotMessage(msgToSend);
					}
				}
			}
		}
		else if ( FStrEq(argv[1], "team") )
		{
			if (argc > 3)
			{
				// Construct the message strings
				char * bot_command = (char *)argv[3];				
				char bot_argument[500];
				
				// Stitch the rest of the string up for the argument string
				if (argc > 4)
					strcpy(bot_argument, (char *)argv[4]);
				else
					strcpy(bot_argument, "*none*");
				for (int j = 5; j < argc; j++)
				{
					strcat(bot_argument, " ");
					strcat(bot_argument, (char *)argv[j]);					
				}
				
				// Send the command to every bot on that team
				for (int i = 0; i < MAX_CLIENTS; i++)
				{
					CBasePlayer * pPlayer = (CBasePlayer *)UTIL_PlayerByIndex(i);
					if (pPlayer && pPlayer->isBot)
					{
						if (UTIL_TeamsMatch(argv[2], pPlayer->TeamID()))
						{
							// Send the message
							CBotMessage * msgToSend = new CBotMessage(GetPlayerIndex(pIssuingPlayer), i, bot_command, bot_argument);
							botMessenger.PostBotMessage(msgToSend);
						}
					}
				}
			}
		}
		else if ( FStrEq(argv[1], "add") )
		{
			// Check how many bots to add (default is 1)
			int numToAdd = 1;
			if (argc > 2)
				numToAdd = atoi(argv[2]);

			if (numToAdd < 1)
				return;

			// Add the bots
			/*for (int i = 0; i < numToAdd; i++){

				BotCreate();
				
			}*/

			// Increase the number of bots waiting to be added (adds are slightly time-delayed to prevent console display buffer overruns)
			if (botsWaitingToBeAdded == 0) {
				BotCreate();
				botsWaitingToBeAdded--;
			}
			botsWaitingToBeAdded += numToAdd;
			
			return;
		}
		else if ( FStrEq(argv[1], "remove") )
		{
			CBasePlayer * pPlayerToKick = (CBasePlayer *)UTIL_PlayerByIndex(atoi(argv[2]));
			if (pPlayerToKick)
			{
				// ClientDisconnect( pPlayerToKick->edict() );
				KickPlayer(atoi(argv[2]));
			}
			return;
		}
/*		else if ( FStrEq(argv[1], "unload") )
		{
			CBasePlayer * pPlayerToUnload = (CBasePlayer *)UTIL_PlayerByIndex(atoi(argv[2]));
			if (pPlayerToUnload)
			{
				if (pPlayerToUnload->isBot)
					((CBot *)pPlayerToUnload)->UnloadBehaviorDll();
			}
		} */
		else if ( FStrEq(argv[1], "about") )
		{
			PrintAboutFlexBot();
			return;
		}
		else if ( FStrEq(argv[1], "help") )
		{
			FlexBotHelp();
			return;
		}
		else if ( FStrEq(argv[1], "menu") )
		{
			// Bail out if this wasn't sent by a player
			if (!pIssuingPlayer)
			{
				BotDebug("Menu command not issued by a valid player.\n");
				return;
			}

			GivePlayerNewMenu(pIssuingPlayer, Menu_Main, "Select an action:", 9);
			PopulateMainMenu(pIssuingPlayer->m_Menu);
			pIssuingPlayer->m_Menu->ShowMenu(pIssuingPlayer);
			return;
		}
		else if ( FStrEq(argv[1], "pause") )
		{
			CBot * pPauseBot;
			if (argc == 2)
			{
				// Pause of unpause depending on whether the first bot is paused
				pPauseBot = FindFirstBot();
				if (pPauseBot)
				{
					if (pPauseBot->m_bPaused)
						UnpauseAllBots();
					else
						PauseAllBots();
					return;
				}
			}
			else if (argc == 3)
			{
				if ( FStrEq(argv[2], "on") )
				{
					PauseAllBots();
					return;
				}
				else if ( FStrEq(argv[2], "off") )
				{
					UnpauseAllBots();
					return;
				}
			}
		}
		else if ( FStrEq(argv[1], "chasecam") )
		{
			// Bail out if this wasn't sent by a player
			if (!pIssuingPlayer)
				return;

			if(argc ==3){
				if ( FStrEq(argv[2], "off") )
				{
					if(pIssuingPlayer->pChaseCam)
						// Leave chase cam mode
						pIssuingPlayer->pChaseCam->Disconnect();

					return;
				}
				else
				{
					CBasePlayer *pPlayerToChase = (CBasePlayer *)UTIL_PlayerByIndex(atoi(argv[2]));
					
					if (!pPlayerToChase)
					{
						BotDebug("There is no player with that client index.");
					}
					else
					{
						// If the player is already using the chase cam, switch the view
						if (pIssuingPlayer->pChaseCam)
							pIssuingPlayer->pChaseCam->ChangeViewToPlayer(pPlayerToChase);
						else
						{
							ClientKill(pIssuingPlayer->edict());

							// Jump into chase cam mode
							pIssuingPlayer->pChaseCam = CChaseCam::Create(pIssuingPlayer, pPlayerToChase);
						}
					}
					return;
				}
			}
			else{
				BotDebug("to use the chasecam enter the bot number of the bot you want to follow or off to  turn it off");
			}
		}
		else if ( FStrEq(argv[1], "freecam") )
		{
			// Bail out if this wasn't sent by a player
			if (!pIssuingPlayer)
				return;

			if (argc ==3){
				if ( FStrEq(argv[2], "off") )
				{
					if(pIssuingPlayer->pFreeCam)
						// Leave chase cam mode
						pIssuingPlayer->pFreeCam->Disconnect();
					return;
				}
				else if (!FStrEq(argv[2], "on"))
				{
					return;
				}
			}
			
			if (pIssuingPlayer)
			{
				// If the player is already using the chase cam, switch the view
				if (pIssuingPlayer->pFreeCam)
				{
					//pIssuingPlayer->pFreeCam->ChangeViewToPlayer(pPlayerToChase);
				}
				else
				{
					ClientKill(pIssuingPlayer->edict());
					
					// Jump into chase cam mode 
					pIssuingPlayer->pFreeCam = CFreeCam::Create(pIssuingPlayer);
				}
			}
			return;
			
		}
		else if ( FStrEq(argv[1], "debug") )
		{
			CBot * pDebugBot;
			if (argc == 2)
			{
				// Toggle the debug based on whatever the first bot's pause is
				// Pause of unpause depending on whether the first bot is paused
				pDebugBot = FindFirstBot();
				if (pDebugBot)
				{
					if (pDebugBot->m_bPaused)
						UnpauseAllBots();
					else
						PauseAllBots();
				}
			}
			else if (argc == 3)
			{
				if ( FStrEq(argv[2], "on") )
				{
					DebugAllBotsOn();
					return;
				}
				else if ( FStrEq(argv[2], "off") )
				{
					DebugAllBotsOff();
					return;
				}
			}
		}
		else if ( FStrEq(argv[1], "tournament") )
		{
			// Load up tournament mode
			if (argc >= 4)
			{
				// Get the team name for team 1
				char team1Name[100];
				bool foundBot = FALSE;
				for (int i = 1; i <= gpGlobals->maxClients; i++ )
				{
					CBasePlayer *pPlayer = (CBasePlayer *)UTIL_PlayerByIndex( i );
					if(pPlayer && pPlayer->isBot)
					{
						strcpy(team1Name, g_pGameRules->GetTeamID(pPlayer));
						foundBot = TRUE;
						break;
					}
				}

				// Make sure a bot was found for the first team
				if (!foundBot)
				{
					BotDebug("Couldn't find any bots for a tournament.\n");
					return;
				}
				else
					foundBot = FALSE;

				// Get the team name for team 2
				char team2Name[100];
				for (i = 1; i <= gpGlobals->maxClients; i++ )
				{
					CBasePlayer *pPlayer = (CBasePlayer *)UTIL_PlayerByIndex( i );
					if(pPlayer && pPlayer->isBot)
					{
						if (!UTIL_TeamsMatch(g_pGameRules->GetTeamID(pPlayer), team1Name))
						{
							strcpy(team2Name, g_pGameRules->GetTeamID(pPlayer));
							foundBot = TRUE;
							break;
						}
						
					}
				}

				// Make sure an opposing bot was found
				if (!foundBot) 
				{
					BotDebug("Couldn't find a bot on the opposing team for a tournament.\n");
					return;
				}

				// Clean up the old tournament (if there was one)
				if (tournament)
					delete tournament;

				// Initialize a tournament
				tournament = new BotTournament(team1Name, argv[2], team2Name, argv[3]);

				// Load up the behavior for team 1
				for (i = 1; i <= gpGlobals->maxClients; i++ )
				{
					CBasePlayer *pPlayer = (CBasePlayer *)UTIL_PlayerByIndex( i );
					if(pPlayer && pPlayer->isBot)
					{
						if (UTIL_TeamsMatch(g_pGameRules->GetTeamID(pPlayer), team1Name))
						{
							char loadCmd[1000];
							sprintf(loadCmd, "bot %d dll %s", i, argv[2]);
							ProcessCommand(pIssuingPlayer, loadCmd);
						}
					}
				}

				// Load up the behaivor for team 2
				for (i = 1; i <= gpGlobals->maxClients; i++ )
				{
					CBasePlayer *pPlayer = (CBasePlayer *)UTIL_PlayerByIndex( i );
					if(pPlayer && pPlayer->isBot)
					{
						if (UTIL_TeamsMatch(g_pGameRules->GetTeamID(pPlayer), team2Name))
						{
							char loadCmd[1000];
							sprintf(loadCmd, "bot %d dll %s", i, argv[3]);
							ProcessCommand(pIssuingPlayer, loadCmd);
						}
					}
				}
			}
		}
		else
		{
			// Treat it as a command to a specific bot or to a group of bots
			int bot_num[MAX_CLIENTS];
			int botNum=0;
			for(int i=1; i < argc ;i++)
			{
				if(isBotNumber(argv[i]))
				{
					//MessageBox(NULL,argv[i],"botNum",MB_OK);
					bot_num[botNum] = atoi(argv[i]);
					botNum++;
				}
				else
					break;	// Stop checking once a non-bot-number is found
			}

			if(botNum + 2 > argc){
				BotDebug("Unrecognized command. Wrong number of arguments.\n");
				return;
			}
			char * bot_command = (char *)argv[botNum + 1];
									
			char bot_argument[500];
			
			if (botNum + 2 < argc)
				strcpy(bot_argument, (char *)argv[botNum + 2]);
			else
				strcpy(bot_argument, "*none*");

			for (int j = botNum + 3; j < argc; j++)
			{
				strcat(bot_argument, " ");
				strcat(bot_argument, (char *)argv[j]);					
			}

			for(i=0; i < botNum ;i++)
			{				
				// Notify the console that we are sending the message
				CBotMessage * msgToSend = new CBotMessage(GetPlayerIndex(pIssuingPlayer), bot_num[i], bot_command, bot_argument);
				
				// Send the message
				botMessenger.PostBotMessage(msgToSend);
			}
			return;
		}
		return;
	}
	BotDebug("Unrecognized command.");
	SERVER_COMMAND((char *)cmd);
	// CLIENT_COMMAND(NULL, (char *)cmd);
}