#include "extdll.h" 
#include "util.h"
#include "cbase.h"
#include "player.h"
#include "bot.h"

#include "FlexControl.h"
#include "BotUtility.h"

#include <io.h>

extern FlexControl * flexControl;

/*---------------------------------------------*
 * FlexControl                                 *
 *---------------------------------------------*/

FlexControl::FlexControl() {
	struct _finddata_t dll_file;
    long hFile;
	char path[1000];

	BuildModFilename(path, "dlls/FlexControl/*.dll");

	// Search for all .dll files in the directory and attempt to load them as FlexControl modules
	if( (hFile = _findfirst( path, &dll_file )) == -1L )
		return; // there are no DLL's to load
	else
	{
		LoadModule(dll_file.name);
		while( _findnext( hFile, &dll_file ) == 0 )
		{			
			LoadModule(dll_file.name);
		}
	}
}

FlexControl::LoadModule(const char * filename) {
	// Get the path of the new DLL to load
	char dllPath[1000];
	BuildModFilename(dllPath, "dlls/FlexControl/");
	strcat(dllPath, filename);
	
	FlexControlModule * module;
	module = new FlexControlModule;
	//	bool success = FALSE;
	bool success = module->LoadModule(dllPath);
	
	// Add this module to the vector if the load succeeded
	char successMsg[1000];
	if (success) {
		// MessageBox(NULL, dllPath, "DLL loaded", MB_OK);
		modules.push_back(module);
		sprintf(successMsg, "FlexControl: Loaded module \"%s\"\n", dllPath);
		BotDebug(successMsg);
	}
	else {
		// MessageBox(NULL, dllPath, "DLL did not load", MB_OK);
		sprintf(successMsg, "FlexControl: Error loading module \"%s\"\n", dllPath);
		BotDebug(successMsg);
	}
}

void FlexControl::CheckForCommands() {
	// Listen on each FlexControl module
	for (int i = 0; i < modules.size(); i++)
		if (modules[i] != NULL)
			modules[i]->ListenForCommands();
}

bool FlexControl::CommandsWaiting() {
	return (commands.size() > 0);
}

const char * FlexControl::GetCommand() {
	std::string command = commands.front();
	commands.pop();
	return command.c_str();
}

void FlexControl::AddNewCommand(const char * command) {
	std::string cmdString = command;
	commands.push(cmdString);
}

/*---------------------------------------------*
 * FlexControlModule                           *
 *---------------------------------------------*/

FlexControlModule::FlexControlModule() {
	initialized = FALSE;
}

bool FlexControlModule::LoadModule(const char * dllPath) {
	char errorMsg[1500];
	
	// Load the DLL
	moduleHandle = FALSE;
	// MessageBox(NULL, dllPath, "DLL Path", MB_OK);
	moduleHandle = LoadLibrary(dllPath);

	// Could the DLL be loaded?
	if (moduleHandle)
	{
		// Load the function pointer for ListenForCommands()
		pfnListen = (PFN_LISTEN) GetProcAddress(moduleHandle, "ListenForCommands");
		if (!pfnListen)
		{
			sprintf(errorMsg, "FlexControl: Error loading function ListenForCommands() in module \"%s\"\n", dllPath);
			BotDebug(errorMsg);
			return FALSE;
		}
	}
	else {
		sprintf(errorMsg, "FlexControl: Error loading module \"%s\"\n", dllPath);
		BotDebug(errorMsg);
		return FALSE;
	}

	// Set initialized to true and return success
	// sprintf(errorMsg, "FlexControl: Loaded module \"%s\"\n", dllPath);
	// BotDebug(errorMsg);
	initialized = TRUE;
	return TRUE;
}

void FlexControlModule::ListenForCommands() {
	if (initialized)
		(pfnListen)();
}

/*------------------------------------------------*
 * FlexBot exports for use by FlexControl modules *
 *------------------------------------------------*/
void EXPORT FLEXCONTROL_SendCommand(const char * command) {
	flexControl->AddNewCommand(command);	
}