#include <windows.h>
#include <vector>
#include <queue>
#include <string>

// Typedef for function pointers to external FlexControl routines
typedef void (* PFN_LISTEN)();

// Function exported for use by FlexControl modules
void EXPORT FLEXCONTROL_SendCommand(const char *);

class FlexControlModule {
public:
	FlexControlModule();				// default constructor
	bool LoadModule(const char *);		// constructor loads the specified DLL

	void ListenForCommands();			// call the module's "listen" function
private:
	bool initialized;					// has the module been initialized?
	HINSTANCE moduleHandle;				// handle to the DLL that is loaded
	PFN_LISTEN pfnListen;				// function pointer for external Reset()
};

class FlexControl {
public:
	FlexControl();				// default constructor loads all available FlexControl modules
	void CheckForCommands();	// calls each module's "listen" function to check for commands
	bool CommandsWaiting();		// are there any commands waiting to be sent?
	const char * GetCommand();	// returns one of the waiting commands
	void AddNewCommand(const char *);	// enqueues a command for execution
private:
	std::vector<FlexControlModule *> modules;		// vector of modules that are loaded
	LoadModule(const char *);						// loads a module by name
	std::queue<std::string> commands;				// queue of waiting commands
};