#include "extdll.h" 
#include "util.h"
#include "cbase.h"
#include "player.h"
#include "bot.h"

#include "FlexDebug.h"
#include "BotUtility.h"

#include <io.h>

/*---------------------------------------------*
 * FlexDebug                                   *
 *---------------------------------------------*/

FlexDebug::FlexDebug() {
	struct _finddata_t dll_file;
    long hFile;
	char path[1000];

	BuildModFilename(path, "dlls/FlexDebug/*.dll");

	// Search for all .dll files in the directory and attempt to load them as FlexDebug modules
	if( (hFile = _findfirst( path, &dll_file )) == -1L )
		return; // there are no DLL's to load
	else
	{
		LoadModule(dll_file.name);
		while( _findnext( hFile, &dll_file ) == 0 )
		{			
			LoadModule(dll_file.name);
		}
	}
}

FlexDebug::LoadModule(const char * filename) {
	// Get the path of the new DLL to load
	char dllPath[1000];
	BuildModFilename(dllPath, "dlls/FlexDebug/");
	strcat(dllPath, filename);
	
	FlexDebugModule * module;
	module = new FlexDebugModule;
	//	bool success = FALSE;
	bool success = module->LoadModule(dllPath);
	
	// Add this module to the vector if the load succeeded
	char successMsg[1000];
	if (success) {
		// MessageBox(NULL, dllPath, "DLL loaded", MB_OK);
		modules.push_back(module);
		sprintf(successMsg, "FlexDebug: Loaded module \"%s\"\n", dllPath);
		BotDebug(successMsg);
	}
	else {
		// MessageBox(NULL, dllPath, "DLL did not load", MB_OK);
		sprintf(successMsg, "FlexDebug: Error loading module \"%s\"\n", dllPath);
		BotDebug(successMsg);
	}
}

FlexDebug::Reset() {
	// Notify each FlexDebug module
	for (int i = 0; i < modules.size(); i++)
		if (modules[i] != NULL)
			modules[i]->Reset();
}

FlexDebug::BotRemoved(int botNum) {
	// Notify each FlexDebug module
	for (int i = 0; i < modules.size(); i++)
		if (modules[i] != NULL)
			modules[i]->BotRemoved(botNum);
}

FlexDebug::DebugMessage(int botNum, const char * msg) {
	// Send the message to each FlexDebug module
	for (int i = 0; i < modules.size(); i++)
		if (modules[i] != NULL)
			modules[i]->DebugMessage(botNum, msg);
}

/*---------------------------------------------*
 * FlexDebugModule                             *
 *---------------------------------------------*/

FlexDebugModule::FlexDebugModule() {
	initialized = FALSE;
}

bool FlexDebugModule::LoadModule(const char * dllPath) {
	char errorMsg[1500];
	
	// Load the DLL
	moduleHandle = FALSE;
	// MessageBox(NULL, dllPath, "DLL Path", MB_OK);
	moduleHandle = LoadLibrary(dllPath);

	// Could the DLL be loaded?
	if (moduleHandle)
	{
		// Load the function pointer for Reset()
		pfnReset = (PFN_RESET) GetProcAddress(moduleHandle, "Reset");
		if (!pfnReset)
		{
			sprintf(errorMsg, "FlexDebug: Error loading function Reset() in module \"%s\"\n", dllPath);
			BotDebug(errorMsg);
			return FALSE;
		}
		
		// Load the function pointer for BotRemoved()
		pfnBotRemoved = (PFN_BOT_REMOVED) GetProcAddress(moduleHandle, "BotRemoved");
		if (!pfnBotRemoved)
		{
			sprintf(errorMsg, "FlexDebug: Error loading function BotRemoved() in module \"%s\"\n", dllPath);
			BotDebug(errorMsg);
			return FALSE;
		}

		// Load the function pointer for DebugMessage()
		pfnDebugMessage = (PFN_DEBUG_MESSAGE) GetProcAddress(moduleHandle, "DebugMessage");
		if (!pfnDebugMessage)
		{
			sprintf(errorMsg, "FlexDebug: Error loading function DebugMessage() in module \"%s\"\n", dllPath);
			BotDebug(errorMsg);
			return FALSE;
		}
	}
	else {
		sprintf(errorMsg, "FlexDebug: Error loading module \"%s\"\n", dllPath);
		BotDebug(errorMsg);
		return FALSE;
	}

	// Set initialized to true and return success
	// sprintf(errorMsg, "FlexDebug: Loaded module \"%s\"\n", dllPath);
	// BotDebug(errorMsg);
	initialized = TRUE;
	return TRUE;
}

void FlexDebugModule::DebugMessage(int botNum, const char * msg) {
	if (initialized)
		(pfnDebugMessage)(botNum, msg);
}

void FlexDebugModule::BotRemoved(int botNum) {
	if (initialized)
		(pfnBotRemoved)(botNum);
}

void FlexDebugModule::Reset() {
	if (initialized)
		(pfnReset)();
}