#include <windows.h>
#include <vector>

// Typedef for function pointers to external FlexDebug routines
typedef void (* PFN_RESET)();
typedef void (* PFN_BOT_REMOVED)(int);
typedef void (* PFN_DEBUG_MESSAGE)(int, const char *);

class FlexDebugModule {
public:
	FlexDebugModule();					// default constructor
	bool LoadModule(const char *);		// constructor loads the specified DLL

	void Reset();						 // called when it's a new game (old bots may no longer exist)
	void BotRemoved(int);				 // called when a bot is removed
	void DebugMessage(int, const char *);// called when a message is sent
private:
	bool initialized;
	HINSTANCE moduleHandle;				// handle to the DLL that is loaded
	PFN_RESET pfnReset;					// function pointer for external Reset()
	PFN_BOT_REMOVED pfnBotRemoved;		// function pointer for external BotRemoved()
	PFN_DEBUG_MESSAGE pfnDebugMessage;	// function pointer for external DebugMessage()	
};

class FlexDebug {
public:
	FlexDebug();		// default constructor loads all available FlexDebug modules
	Reset();			// notify the debugger that it's a new game (old bots may no longer exist)
	BotRemoved(int);	// notify the debugger that a bot has been removed
	DebugMessage(int, const char *);	// send a debug message to all loaded modules
private:
	std::vector<FlexDebugModule *> modules;		// vector of modules that are loaded
	LoadModule(const char *);
};