#include "extdll.h" 
#include "util.h"
#include "cbase.h"
#include "player.h"
#include "weapons.h"
#include "gamerules.h"
#include "bot.h"

#include "FlexMonitor.h"
#include "BotUtility.h"

#include <io.h>

/*---------------------------------------------*
 * FlexMonitor                                 *
 *---------------------------------------------*/

FlexMonitor::FlexMonitor() {
	struct _finddata_t dll_file;
    long hFile;
	char path[1000];

	BuildModFilename(path, "dlls/FlexMonitor/*.dll");

	// Search for all .dll files in the directory and attempt to load them as FlexControl modules
	if( (hFile = _findfirst( path, &dll_file )) == -1L )
		return; // there are no DLL's to load
	else
	{
		LoadModule(dll_file.name);
		while( _findnext( hFile, &dll_file ) == 0 )
		{			
			LoadModule(dll_file.name);
		}
	}
}

FlexMonitor::LoadModule(const char * filename) {
	// Get the path of the new DLL to load
	char dllPath[1000];
	BuildModFilename(dllPath, "dlls/FlexMonitor/");
	strcat(dllPath, filename);
	
	FlexMonitorModule * module;
	module = new FlexMonitorModule;
	//	bool success = FALSE;
	bool success = module->LoadModule(dllPath);
	
	// Add this module to the vector if the load succeeded
	char successMsg[1000];
	if (success) {
		// MessageBox(NULL, dllPath, "DLL loaded", MB_OK);
		modules.push_back(module);
		sprintf(successMsg, "FlexMonitor: Loaded module \"%s\"\n", dllPath);
		BotDebug(successMsg);
	}
	else {
		// MessageBox(NULL, dllPath, "DLL did not load", MB_OK);
		sprintf(successMsg, "FlexMonitor: Error loading module \"%s\"\n", dllPath);
		BotDebug(successMsg);
	}
}

void FlexMonitor::PlayerUpdate(int playerNum, int x, int y, int z, int health, float facing, float pitch, const char * weapon) {
	// Update each FlexMonitor module
	for (int i = 0; i < modules.size(); i++)
		if (modules[i] != NULL)
			modules[i]->PlayerUpdate(playerNum, x, y, z, health, facing, pitch, weapon);
}


/*---------------------------------------------*
 * FlexMonitorModule                           *
 *---------------------------------------------*/

FlexMonitorModule::FlexMonitorModule() {
	initialized = FALSE;
}

bool FlexMonitorModule::LoadModule(const char * dllPath) {
	char errorMsg[1500];
	
	// Load the DLL
	moduleHandle = FALSE;
	// MessageBox(NULL, dllPath, "DLL Path", MB_OK);
	moduleHandle = LoadLibrary(dllPath);

	// Could the DLL be loaded?
	if (moduleHandle)
	{
		// Load the function pointer for PlayerUpdate()
		pfnUpdate = (PFN_UPDATE) GetProcAddress(moduleHandle, "PlayerUpdate");
		if (!pfnUpdate)
		{
			sprintf(errorMsg, "FlexMonitor: Error loading function PlayerUpdate() in module \"%s\"\n", dllPath);
			BotDebug(errorMsg);
			return FALSE;
		}
	}
	else {
		sprintf(errorMsg, "FlexMonitor: Error loading module \"%s\"\n", dllPath);
		BotDebug(errorMsg);
		return FALSE;
	}

	// Set initialized to true and return success
	// sprintf(errorMsg, "FlexControl: Loaded module \"%s\"\n", dllPath);
	// BotDebug(errorMsg);
	initialized = TRUE;
	return TRUE;
}

void FlexMonitorModule::PlayerUpdate(int playerNum, int x, int y, int z, int health, float facing, float pitch, const char * weapon) {
	if (initialized)
		(pfnUpdate)(playerNum, x, y, z, health, facing, pitch, weapon);
}

/*------------------------------------------------*
 * FlexBot exports for use by FlexMonitor modules *
 *------------------------------------------------*/
const char * EXPORT FLEXMONITOR_CurrentMap() {
	return STRING(gpGlobals->mapname);
}

bool EXPORT FLEXMONITOR_IsPlayerBot(int playerNum) {
	CBasePlayer * pPlayer = (CBasePlayer *)UTIL_PlayerByIndex(playerNum);
	return (pPlayer && pPlayer->isBot);
}

const char * EXPORT FLEXMONITOR_BotBehaviorName(int playerNum) {
	CBasePlayer * pPlayer = (CBasePlayer *)UTIL_PlayerByIndex(playerNum);
	if (pPlayer && pPlayer->isBot) {
		return ((CBot *)pPlayer)->behaviorName;
	}
	else
		return "*none*";
}

const char * EXPORT FLEXMONITOR_PlayerTeam(int playerNum) {
	CBasePlayer * pPlayer = (CBasePlayer *)UTIL_PlayerByIndex(playerNum);
	if (pPlayer)
		return pPlayer->TeamID();
	else
		return "*none*";
}

const char * EXPORT FLEXMONITOR_GameType() {
	if (g_pGameRules->IsTeamplay())
		return "teamplay";
	else
		return "deathmatch";
}

const char * EXPORT FLEXMONITOR_PlayerName(int playerNum) {
	CBasePlayer * pPlayer = (CBasePlayer *)UTIL_PlayerByIndex(playerNum);
	if (pPlayer)
		return STRING(pPlayer->pev->netname);
	else
		return "*none*";
}

int EXPORT FLEXMONITOR_PlayerKills(int playerNum) {
	CBasePlayer * pPlayer = (CBasePlayer *)UTIL_PlayerByIndex(playerNum);
	if (pPlayer)
		return ((CBot *)pPlayer)->statKills;
	else
		return 0;
}

int EXPORT FLEXMONITOR_PlayerDeaths(int playerNum) {
	CBasePlayer * pPlayer = (CBasePlayer *)UTIL_PlayerByIndex(playerNum);
	if (pPlayer)
		return ((CBot *)pPlayer)->statDeaths;
	else
		return 0;
}