#include <windows.h>
#include <vector>
#include <queue>

// Typedef for function pointers to external FlexMonitor routines
typedef void (* PFN_UPDATE)(int, int, int, int, int, float, float, const char *);

// Function exported for use by FlexControl modules
const char * EXPORT FLEXMONITOR_CurrentMap();
bool EXPORT FLEXMONITOR_IsPlayerBot(int);
const char * EXPORT FLEXMONITOR_BotBehaviorName(int);
const char * EXPORT FLEXMONITOR_PlayerTeam(int);
const char * EXPORT FLEXMONITOR_GameType();
const char * EXPORT FLEXMONITOR_PlayerName(int);
int EXPORT FLEXMONITOR_PlayerKills(int);
int EXPORT FLEXMONITOR_PlayerDeaths(int);

class FlexMonitorModule {
public:
	FlexMonitorModule();				// default constructor
	bool LoadModule(const char *);		// constructor loads the specified DLL

	void PlayerUpdate(int, int, int, int, int, float, float, const char *);	// call the module's "update" function
private:
	bool initialized;					// has the module been initialized?
	HINSTANCE moduleHandle;				// handle to the DLL that is loaded
	PFN_UPDATE pfnUpdate;				// function pointer for external Update()
};

class FlexMonitor {
public:
	FlexMonitor();				// default constructor loads all available FlexMonitor modules
	void PlayerUpdate(int, int, int, int, int, float, float, const char *);		// calls each module's "update" function with new state information
private:
	std::vector<FlexMonitorModule *> modules;		// vector of modules that are loaded
	LoadModule(const char *);						// loads a module by name
};