#include "extdll.h" 
#include "util.h"
#include "cbase.h"
#include "player.h"
#include "weapons.h"
#include "gamerules.h"
#include "bot.h"

#include "FlexStat.h"
#include "BotUtility.h"

#include <io.h>

/*---------------------------------------------*
 * FlexStat                                    *
 *---------------------------------------------*/

FlexStat::FlexStat() {
	struct _finddata_t dll_file;
    long hFile;
	char path[1000];

	BuildModFilename(path, "dlls/FlexStat/*.dll");

	// Search for all .dll files in the directory and attempt to load them as FlexControl modules
	if( (hFile = _findfirst( path, &dll_file )) == -1L )
		return; // there are no DLL's to load
	else
	{
		LoadModule(dll_file.name);
		while( _findnext( hFile, &dll_file ) == 0 )
		{			
			LoadModule(dll_file.name);
		}
	}
}

FlexStat::LoadModule(const char * filename) {
	// Get the path of the new DLL to load
	char dllPath[1000];
	BuildModFilename(dllPath, "dlls/FlexStat/");
	strcat(dllPath, filename);
	
	FlexStatModule * module;
	module = new FlexStatModule;
	//	bool success = FALSE;
	bool success = module->LoadModule(dllPath);
	
	// Add this module to the vector if the load succeeded
	char successMsg[1000];
	if (success) {
		// MessageBox(NULL, dllPath, "DLL loaded", MB_OK);
		modules.push_back(module);
		sprintf(successMsg, "FlexStat: Loaded module \"%s\"\n", dllPath);
		BotDebug(successMsg);
	}
	else {
		// MessageBox(NULL, dllPath, "DLL did not load", MB_OK);
		sprintf(successMsg, "FlexStat: Error loading module \"%s\"\n", dllPath);
		BotDebug(successMsg);
	}
}

void FlexStat::AddPlayer(int playerNum) {
	// AddPlayer for each FlexStat module
	for (int i = 0; i < modules.size(); i++)
		if (modules[i] != NULL)
			modules[i]->AddPlayer(playerNum);
}

void FlexStat::RemovePlayer(int playerNum) {
	// RemovePlayer for each FlexStat module
	for (int i = 0; i < modules.size(); i++)
		if (modules[i] != NULL)
			modules[i]->RemovePlayer(playerNum);
}

void FlexStat::UpdatePlayerName(int playerNum, const char * playerName) {
	// UpdatePlayerName for each FlexStat module
	for (int i = 0; i < modules.size(); i++)
		if (modules[i] != NULL)
			modules[i]->UpdatePlayerName(playerNum, playerName);
}

void FlexStat::UpdatePlayerTeam(int playerNum, const char * playerTeam) {
	// UpdatePlayerTeam for each FlexStat module
	for (int i = 0; i < modules.size(); i++)
		if (modules[i] != NULL)
			modules[i]->UpdatePlayerTeam(playerNum, playerTeam);
}

void FlexStat::UpdateBehavior(int playerNum, const char * behaviorName) {
	// UpdateBehavior for each FlexStat module
	for (int i = 0; i < modules.size(); i++)
		if (modules[i] != NULL)
			modules[i]->UpdateBehavior(playerNum, behaviorName);
}

void FlexStat::GameStart() {
	// GameStart for each FlexStat module
	for (int i = 0; i < modules.size(); i++)
		if (modules[i] != NULL)
			modules[i]->GameStart();
}

void FlexStat::UpdatePlayerStat(int playerNum, const char * statName, const char * statValue) {
	// UpdatePlayerStat for each FlexStat module
	for (int i = 0; i < modules.size(); i++)
		if (modules[i] != NULL)
			modules[i]->UpdatePlayerStat(playerNum, statName, statValue);
}

void FlexStat::UpdateTeamStat(const char * teamName, const char * statName, const char * statValue) {
	// UpdateTeamStat for each FlexStat module
	for (int i = 0; i < modules.size(); i++)
		if (modules[i] != NULL)
			modules[i]->UpdateTeamStat(teamName, statName, statValue);
}

void FlexStat::UpdateGameStat(const char * statName, const char * statValue) {
	// UpdateGameStat for each FlexStat module
	for (int i = 0; i < modules.size(); i++)
		if (modules[i] != NULL)
			modules[i]->UpdateGameStat(statName, statValue);
}


/*---------------------------------------------*
 * FlexStatModule                           *
 *---------------------------------------------*/

FlexStatModule::FlexStatModule() {
	initialized = FALSE;
}

bool FlexStatModule::LoadModule(const char * dllPath) {
	char errorMsg[1500];
	
	// Load the DLL
	moduleHandle = FALSE;
	// MessageBox(NULL, dllPath, "DLL Path", MB_OK);
	moduleHandle = LoadLibrary(dllPath);

	// Could the DLL be loaded?
	if (moduleHandle)
	{
		// Load the function pointer for AddPlayer()
		pfnAddPlayer = (PFN_ADD_PLAYER) GetProcAddress(moduleHandle, "AddPlayer");
		if (!pfnAddPlayer)
		{
			sprintf(errorMsg, "FlexStat: Error loading function AddPlayer() in module \"%s\"\n", dllPath);
			BotDebug(errorMsg);
			return FALSE;
		}

		// Load the function pointer for RemovePlayer()
		pfnRemovePlayer = (PFN_REMOVE_PLAYER) GetProcAddress(moduleHandle, "RemovePlayer");
		if (!pfnRemovePlayer)
		{
			sprintf(errorMsg, "FlexStat: Error loading function RemovePlayer() in module \"%s\"\n", dllPath);
			BotDebug(errorMsg);
			return FALSE;
		}

		// Load the function pointer for UpdatePlayerName()
		pfnUpdatePlayerName = (PFN_UPDATE_PLAYER_NAME) GetProcAddress(moduleHandle, "UpdatePlayerName");
		if (!pfnUpdatePlayerName)
		{
			sprintf(errorMsg, "FlexStat: Error loading function UpdatePlayerName() in module \"%s\"\n", dllPath);
			BotDebug(errorMsg);
			return FALSE;
		}

		// Load the function pointer for UpdatePlayerTeam()
		pfnUpdatePlayerTeam = (PFN_UPDATE_PLAYER_TEAM) GetProcAddress(moduleHandle, "UpdatePlayerTeam");
		if (!pfnUpdatePlayerTeam)
		{
			sprintf(errorMsg, "FlexStat: Error loading function UpdatePlayerTeam() in module \"%s\"\n", dllPath);
			BotDebug(errorMsg);
			return FALSE;
		}

		// Load the function pointer for UpdateBotBehavior()
		pfnUpdateBotBehavior = (PFN_UPDATE_BOT_BEHAVIOR) GetProcAddress(moduleHandle, "UpdateBotBehavior");
		if (!pfnUpdateBotBehavior)
		{
			sprintf(errorMsg, "FlexStat: Error loading function UpdateBotBehavior() in module \"%s\"\n", dllPath);
			BotDebug(errorMsg);
			return FALSE;
		}

		// Load the function pointer for GameStart()
		pfnGameStart = (PFN_GAME_START) GetProcAddress(moduleHandle, "GameStart");
		if (!pfnGameStart)
		{
			sprintf(errorMsg, "FlexStat: Error loading function GameStart() in module \"%s\"\n", dllPath);
			BotDebug(errorMsg);
			return FALSE;
		}

		// Load the function pointer for UpdatePlayerStat()
		pfnUpdatePlayerStat = (PFN_UPDATE_PLAYER_STAT) GetProcAddress(moduleHandle, "UpdatePlayerStat");
		if (!pfnUpdatePlayerStat)
		{
			sprintf(errorMsg, "FlexStat: Error loading function UpdatePlayerStat() in module \"%s\"\n", dllPath);
			BotDebug(errorMsg);
			return FALSE;
		}

		// Load the function pointer for UpdateTeamStat()
		pfnUpdateTeamStat = (PFN_UPDATE_TEAM_STAT) GetProcAddress(moduleHandle, "UpdateTeamStat");
		if (!pfnUpdateTeamStat)
		{
			sprintf(errorMsg, "FlexStat: Error loading function UpdateTeamStat() in module \"%s\"\n", dllPath);
			BotDebug(errorMsg);
			return FALSE;
		}

		// Load the function pointer for UpdateGameStat()
		pfnUpdateGameStat = (PFN_UPDATE_GAME_STAT) GetProcAddress(moduleHandle, "UpdateGameStat");
		if (!pfnUpdateGameStat)
		{
			sprintf(errorMsg, "FlexStat: Error loading function UpdateGameStat() in module \"%s\"\n", dllPath);
			BotDebug(errorMsg);
			return FALSE;
		}
	}
	else {
		sprintf(errorMsg, "FlexStat: Error loading module \"%s\"\n", dllPath);
		BotDebug(errorMsg);
		return FALSE;
	}

	// Set initialized to true and return success
	// sprintf(errorMsg, "FlexControl: Loaded module \"%s\"\n", dllPath);
	// BotDebug(errorMsg);
	initialized = TRUE;
	return TRUE;
}

void FlexStatModule::AddPlayer(int playerNum) {
	if (initialized)
		pfnAddPlayer(playerNum);
}

void FlexStatModule::RemovePlayer(int playerNum) {
	if (initialized)
		pfnRemovePlayer(playerNum);
}

void FlexStatModule::UpdatePlayerName(int playerNum, const char * playerName) {
	if (initialized)
		pfnUpdatePlayerName(playerNum, playerName);
}

void FlexStatModule::UpdatePlayerTeam(int playerNum, const char * playerTeam) {
	if (initialized)
		pfnUpdatePlayerTeam(playerNum, playerTeam);
}

void FlexStatModule::UpdateBehavior(int playerNum, const char * behaviorName) {
	if (initialized)
		pfnUpdateBotBehavior(playerNum, behaviorName);
}

void FlexStatModule::GameStart() {
	if (initialized)
		pfnGameStart();
}

void FlexStatModule::UpdatePlayerStat(int playerNum, const char * statName, const char * statValue) {
	if (initialized)
		pfnUpdatePlayerStat(playerNum, statName, statValue);
}

void FlexStatModule::UpdateTeamStat(const char * teamName, const char * statName, const char * statValue) {
	if (initialized)
		pfnUpdateTeamStat(teamName, statName, statValue);
}

void FlexStatModule::UpdateGameStat(const char * statName, const char * statValue) {
	if (initialized)
		pfnUpdateGameStat(statName, statValue);
}