#include <windows.h>
#include <vector>

// Typedef for function pointers to external FlexMonitor routines
typedef void (* PFN_ADD_PLAYER)(int);
typedef void (* PFN_REMOVE_PLAYER)(int);
typedef void (* PFN_UPDATE_PLAYER_NAME)(int, const char *);
typedef void (* PFN_UPDATE_PLAYER_TEAM)(int, const char *);
typedef void (* PFN_UPDATE_BOT_BEHAVIOR)(int, const char *);
typedef void (* PFN_GAME_START)();
typedef void (* PFN_UPDATE_PLAYER_STAT)(int, const char *, const char *);
typedef void (* PFN_UPDATE_TEAM_STAT)(const char *, const char *, const char *);
typedef void (* PFN_UPDATE_GAME_STAT)(const char *, const char *);

class FlexStatModule {
public:
	FlexStatModule();				// default constructor
	bool LoadModule(const char *);	// constructor loads the specified DLL

	void AddPlayer(int);			// call the module's "update" function
	void RemovePlayer(int);
	void UpdatePlayerName(int, const char *);
	void UpdatePlayerTeam(int, const char *);
	void UpdateBehavior(int, const char *);
	void GameStart();
	void UpdatePlayerStat(int, const char *, const char *);
	void UpdateTeamStat(const char *, const char *, const char *);
	void UpdateGameStat(const char *, const char *);

private:
	bool initialized;					// has the module been initialized?
	HINSTANCE moduleHandle;				// handle to the DLL that is loaded
	PFN_ADD_PLAYER pfnAddPlayer;		// function pointer for external AddPlayer()
	PFN_REMOVE_PLAYER pfnRemovePlayer;
	PFN_UPDATE_PLAYER_NAME pfnUpdatePlayerName;
	PFN_UPDATE_PLAYER_TEAM pfnUpdatePlayerTeam;
	PFN_UPDATE_BOT_BEHAVIOR pfnUpdateBotBehavior;
	PFN_GAME_START pfnGameStart;
	PFN_UPDATE_PLAYER_STAT pfnUpdatePlayerStat;
	PFN_UPDATE_TEAM_STAT pfnUpdateTeamStat;
	PFN_UPDATE_GAME_STAT pfnUpdateGameStat;
};

class FlexStat {
public:
	FlexStat();				// default constructor loads all available FlexStat modules
	void AddPlayer(int);
	void RemovePlayer(int);
	void UpdatePlayerName(int, const char *);
	void UpdatePlayerTeam(int, const char *);
	void UpdateBehavior(int, const char *);
	void GameStart();
	void UpdatePlayerStat(int, const char *, const char *);
	void UpdateTeamStat(const char *, const char *, const char *);
	void UpdateGameStat(const char *, const char *);
private:
	std::vector<FlexStatModule *> modules;			// vector of modules that are loaded
	LoadModule(const char *);						// loads a module by name
};
