/*********************************************
/ FreeCam.cpp
/
/ This file contains code to run the freelooking
/camera this is based off te ChaseCam system
*********************************************/

#include "extdll.h"
#include "util.h"
#include "client.h"
#include "cbase.h"
#include "player.h"

#include "BotUtility.h"
#include "FreeCam.h"
#include "bot.h"


LINK_ENTITY_TO_CLASS(entity_freecam, CFreeCam);


CFreeCam *CFreeCam::Create( CBasePlayer *pPlayer) //, CBasePlayer *pTarget )
{
	if(pPlayer->pChaseCam){
		pPlayer->pev->origin = pPlayer->pChaseCam->pev->origin;
		pPlayer->pChaseCam->Disconnect();
	}

	CFreeCam * pFreeCam = GetClassPtr( (CFreeCam *)NULL );
	//the invisibke model to be used for teh camera
	PRECACHE_MODEL( "models/mechgibs.mdl" );

	pFreeCam->m_pTarget = NULL;
	pFreeCam->m_pPlayer = pPlayer;
	//pFreeCam->m_pTarget = pTarget;
	//set the player up to not be seen
	pPlayer->pev->effects |= EF_NODRAW;
	pPlayer->pev->solid = SOLID_NOT;
	pPlayer->pev->takedamage = DAMAGE_NO;
	pPlayer->pev->movetype =  MOVETYPE_NOCLIP;
	pPlayer->pev->movetype |= MOVETYPE_FLY;
	// pPlayer->m_iHideHUD |= HIDEHUD_ALL;
	pPlayer->pev->gravity = 0;
	pFreeCam->pev->gravity = 0;

	// Hide everything except for chat messages and menus
	pPlayer->m_iHideHUD |= HIDEHUD_HEALTH;
	pPlayer->m_iHideHUD |= HIDEHUD_WEAPONS;

	pFreeCam->Spawn();

	return pFreeCam;
}
   

void CFreeCam::Spawn( void )
{
	if (!m_pPlayer)
		return;

	pev->classname = MAKE_STRING("entity_freecam");
	// Construct a vector from the origin of following length
	Vector cameraView (CHASECAM_DIST, 0, 0);

	float rotateAngle = /*m_pTarget->pev->v_angle.y + */ m_pPlayer->pev->v_angle.y;
	cameraView = BotUTIL_RotateVectorZ(cameraView, rotateAngle);

	// Set the camera's position to the place where the player was
	UTIL_SetOrigin(pev, m_pPlayer->pev->origin);

	// Set the camera's view angle
	//play with this to get it right
	pev->angles.y = UTIL_VecToAngles(m_pPlayer->pev->origin + m_pPlayer->pev->view_ofs).y;
	pev->fixangle = TRUE;

	SET_VIEW (m_pPlayer->edict(), edict());

	// Set the camera's model to be an invisible model
	SET_MODEL(ENT(pev), "models/mechgibs.mdl");
	//pev->movetype =  MOVETYPE_NOCLIP;
	pev->movetype = MOVETYPE_FLY;
	pev->solid = SOLID_NOT;
	pev->takedamage = DAMAGE_NO;
	pev->renderamt = 0;
	pev->gravity = 0;
	m_pPlayer->pev->gravity = 0;

	m_pPlayer->EnableControl(TRUE);

	SetTouch( NULL );
	SetThink( IdleThink );

	// Tell the player who they are looking at
	//UTIL_ShowMessage(UTIL_VarArgs("Chase cam is following %s\n", STRING(m_pTarget->pev->netname)), m_pPlayer);

	pev->nextthink = gpGlobals->time + 0.1;
}

void CFreeCam::ViewNextPlayer()
{
	int currentTarget;
	if(m_pTarget)
		currentTarget = GetPlayerIndex(m_pTarget);
	else 
		currentTarget =1;
	int goodTarget = currentTarget;
	int loopBackTarget = -1;

	// Find the next target (or loop back)
	for (int newTarget = 1; newTarget <= gpGlobals->maxClients; newTarget++)
	{
		// Don't check the current target
		if (newTarget == currentTarget)
			continue;

		CBasePlayer * pNewPlayer = (CBasePlayer *)UTIL_PlayerByIndex( newTarget );

		// Skip invalid players
		if (!pNewPlayer)
			continue;

		// Don't watch players in chase cam mode
		if (pNewPlayer->pChaseCam)
			continue;

		// Skip this player if dead or dying
		if (pNewPlayer->pev->deadflag != DEAD_NO)
			continue;

		if (loopBackTarget == -1)
			loopBackTarget = newTarget;

		// Store this player as the last good one
		goodTarget = newTarget;

		// Stop looking if the client number is greater than the current target
		if (newTarget > currentTarget)
			break;
	}

	if (goodTarget < currentTarget)
		goodTarget = loopBackTarget;

	// Set the new target
	ChangeViewToPlayer( (CBasePlayer *)UTIL_PlayerByIndex( goodTarget ) );
}

void CFreeCam::ViewPreviousPlayer()
{
	int currentTarget;
	if(m_pTarget)
		 currentTarget = GetPlayerIndex(m_pTarget);
	else 
		 currentTarget =1;
	int goodTarget = currentTarget;
	int loopBackTarget = -1;

	// Find the next target (or loop back)
	for (int newTarget = gpGlobals->maxClients; newTarget >= 1; newTarget--)
	{
		// Don't check the current target
		if (newTarget == currentTarget)
			continue;

		CBasePlayer * pNewPlayer = (CBasePlayer *)UTIL_PlayerByIndex( newTarget );

		// Skip invalid players
		if (!pNewPlayer)
			continue;

		// Don't watch players in chase cam mode
		if (pNewPlayer->pChaseCam)
			continue;

		// Skip this player if dead or dying
		if (pNewPlayer->pev->deadflag != DEAD_NO)
			continue;

		if (loopBackTarget == -1)
			loopBackTarget = newTarget;

		// Store this player as the last good one
		goodTarget = newTarget;

		// Stop looking if the client number is greater than the current target
		if (newTarget < currentTarget)
			break;
	}

	if (goodTarget > currentTarget)
		goodTarget = loopBackTarget;

	// Set the new target
	ChangeViewToPlayer( (CBasePlayer *)UTIL_PlayerByIndex( goodTarget ) );
}

void CFreeCam::ChangeViewToPlayer(CBasePlayer * pTarget)
{
	// Set the new target
	m_pTarget = pTarget;

	UTIL_SetOrigin(pev, m_pTarget->pev->origin+ Vector(-50,0,0));
	// Tell the player who they are looking at
	UTIL_ShowMessage(UTIL_VarArgs("Free cam is near %s\n", STRING(pTarget->pev->netname)), m_pPlayer);
	//UTIL_CenterPrintAll(UTIL_VarArgs("Chase cam is following %s\n", STRING(pTarget->pev->netname)));
}


void CFreeCam::IdleThink( void )
{
	if (!m_pPlayer)
		return;

	// Check if the user wants to cycle to the NEXT player target
	if (!(m_pPlayer->pev->button & IN_ATTACK))
	{
		// Reactivate player switching (so that holding down the
		// button doesn't cycle rapidly)
		bCanSwitchTarget = TRUE;
	}
	else if ((m_pPlayer->pev->button & IN_ATTACK) && bCanSwitchTarget)
	{
		ViewNextPlayer();
		// Deactivate player switching until button is released
		bCanSwitchTarget = FALSE;
	}

	// Check if the user wants to cycle to the PREVIOUS target
	if (!(m_pPlayer->pev->button & IN_ATTACK2))
	{
		// Reactivate player switching (so that holding down the
		// button doesn't cycle rapidly
		bCanSwitchTargetPrev = TRUE;
	}
	else if ((m_pPlayer->pev->button & IN_ATTACK2) && bCanSwitchTargetPrev)
	{
		ViewPreviousPlayer();

		// Deactivate player switching until button is released
		bCanSwitchTargetPrev = FALSE;
	}

	// If the target is still in the game, then view it
	if (m_pPlayer->pev->takedamage == DAMAGE_NO)  // not kicked
	{ 
		UTIL_MakeVectors(m_pPlayer->pev->v_angle);

		// Set the camera's view angle
		pev->angles.y = m_pPlayer->pev->v_angle.y; 
		pev->angles.x = m_pPlayer->pev->v_angle.x;
		pev->angles.z = m_pPlayer->pev->v_angle.z;
		pev->fixangle = TRUE;
		//Vector up;
		int up =0;
		int forward = 0;
		int strafe =0;
		if (m_pPlayer->pev->button & IN_JUMP)
			up= 115;
		if ( m_pPlayer->m_afButtonPressed & IN_FORWARD)
			forward =250;
		if ( m_pPlayer->m_afButtonPressed & IN_BACK)
			forward =-250;
		if ( m_pPlayer->m_afButtonPressed & IN_MOVELEFT)
			strafe = 200;
		if ( m_pPlayer->m_afButtonPressed & IN_MOVERIGHT)
			strafe = -200;

		Vector strafing(cos(degToRad(pev->angles.y+90))*strafe,sin(degToRad(pev->angles.y +90))*strafe,up);

		Vector vecdir(cos(degToRad(pev->angles.y))*forward,
			sin(degToRad(pev->angles.y))*forward,
			cos(degToRad(pev->angles.x + 90))*forward );

		pev->velocity =  vecdir + strafing;
		pev->nextthink = gpGlobals->time;
	}
	else
	{
		// Get rid of the chase cam
		SET_VIEW (m_pPlayer->edict(), m_pPlayer->edict());
		m_pPlayer->pev->effects &= ~EF_NODRAW;
		m_pPlayer->pev->solid = SOLID_SLIDEBOX;
		m_pPlayer->pev->takedamage = DAMAGE_AIM;
		m_pPlayer->m_iHideHUD &= ~HIDEHUD_ALL;
		m_pPlayer->m_iHideHUD &= ~HIDEHUD_HEALTH;
		m_pPlayer->m_iHideHUD &= ~HIDEHUD_WEAPONS;
		m_pPlayer->EnableControl(TRUE);
		m_pPlayer->pFreeCam = NULL;  // player's chase cam is no longer valid
	//	m_pTarget = NULL;
		m_pPlayer = NULL;
		REMOVE_ENTITY( ENT(pev) );
	}
}


void CFreeCam::Disconnect( void )
{
	SET_VIEW (m_pPlayer->edict(), m_pPlayer->edict());

	m_pPlayer->pev->effects &= ~EF_NODRAW;
	m_pPlayer->pev->solid = SOLID_SLIDEBOX;
	m_pPlayer->pev->takedamage = DAMAGE_AIM;
	m_pPlayer->m_iHideHUD &= ~HIDEHUD_ALL;
	m_pPlayer->m_iHideHUD &= ~HIDEHUD_HEALTH;
	m_pPlayer->m_iHideHUD &= ~HIDEHUD_WEAPONS;

	m_pPlayer->EnableControl(TRUE);

	m_pPlayer->pFreeCam = NULL;  // player's chase cam is no longer valid

	m_pTarget = NULL;
	m_pPlayer = NULL;

	REMOVE_ENTITY( ENT(pev) );
}

