#include "extdll.h" 
#include "util.h"
#include "cbase.h"
#include "player.h"
#include "bot.h"

#include "stdlib.h"
#include "string.h"
#include <io.h>

#include "Menus.h"
#include "BotUtility.h"

extern int gmsgShowMenu;	 // the message # for ShowMenu
extern int botBeingCommanded;

// for calculating menu bitmasks...
int menubit_matrix[] = {	0x001, 0x002, 0x004, 0x008,
							0x010, 0x020, 0x040, 0x080,
							0x100, 0x200, 0x400, 0x800	};


void ShowHlMenu (CBasePlayer *pPlayer, int bitsValidSlots, int nDisplayTime, BOOL fNeedMore, char *pszText)
{
    MESSAGE_BEGIN( MSG_ONE, gmsgShowMenu, NULL, pPlayer->pev);
        WRITE_SHORT( bitsValidSlots);
        WRITE_CHAR( nDisplayTime );
        WRITE_BYTE( fNeedMore );
        WRITE_STRING (pszText);
    MESSAGE_END();
}


BotMenuItem::BotMenuItem(const char * itemName, const char * returnValue)
{
	strcpy(name, itemName);
	strcpy(returnVal, returnValue);
}

BotMenu::BotMenu(_Menu ID, const char * title, int pageSize) {
	menuID = ID;
	currentPage = 1;
	numItems = 0;
	pageLength = pageSize;
	strcpy(menuTitle, title);
}

void BotMenu::AddItem(const char * itemName, const char * returnValue)
{
	// if there is room, add the new menu item
	if (numItems < MAX_MENU_ITEMS)
	{
		items[numItems] = new BotMenuItem(itemName, returnValue);
		numItems++;
	}
}

void BotMenu::ClearMenu()
{
	// step through and delete each menu object
	for (int i = 0; i < numItems; i++)
	{
		delete items[i];
	}
	numItems = 0;
}

void BotMenu::InitMenu(const char * title, int setPageLength)
{
	strcpy(menuTitle, title);
	pageLength = setPageLength;
	currentPage = 1;
}

void BotMenu::NextPage()
{
	currentPage++;
}

void BotMenu::ShowMenu(CBasePlayer * pPlayer)
{
	char menuText[MAX_ITEMNAME_LENGTH * (MAX_MENU_ITEMS + 1)];

	// Set the player's current menu to this menu
	pPlayer->m_nMenu = menuID;

	// Give the menu a title
	sprintf(menuText, "%s\n\n", menuTitle);

	// Clear the selection bitmask (no activated selections)
	bitmask = 0x0;
	
	// Construct the menu from the array
	int currentPos = (currentPage - 1) * pageLength;
	for (int i = currentPos; i < currentPos + pageLength; i++)
	{
		if (i < numItems)
		{
			char optionNum[4];
			itoa(i - currentPos + 1, optionNum, 10);
			strcat(menuText, optionNum);
			strcat(menuText, ". ");
			strcat(menuText, items[i]->name);
			strcat(menuText, "\n");
			ActivateSlot(i - currentPos + 1);
		}
	}

	// Is there another "page" of items?
	if (numItems > currentPage * pageLength)
	{
		strcat(menuText, "\n0. More...");
		ActivateSlot(10);
	}
	
	// Show the menu, breaking it up into chunks of 127 character if necessary
	while (strlen(menuText) > 127)
	{
		char menuTextPiece[128];
		strncpy(menuTextPiece, menuText, 127);
		ShowHlMenu(pPlayer, bitmask, -1, true, menuTextPiece);
		char tempMenuText[MAX_ITEMNAME_LENGTH * (MAX_MENU_ITEMS + 1)];
		strcpy(tempMenuText, menuText + 127);
		strcpy(menuText, tempMenuText);
	}
	ShowHlMenu(pPlayer, bitmask, -1, false, menuText);
}

char * BotMenu::GetReturnValue(int itemIndex){
	int menuIndex = (currentPage - 1) * pageLength + itemIndex - 1;
	if (menuIndex < numItems)
		return items[menuIndex]->returnVal;
	else
		return "ERROR: invalid menu selection";
}

void BotMenu::ActivateSlot(int slot)
{
	bitmask += menubit_matrix[slot - 1];
}

void GivePlayerNewMenu(CBasePlayer * pPlayer, _Menu menuID, const char * title, int pageLength)
{
	// Clear out any old menu
	if (pPlayer->m_Menu)
		delete pPlayer->m_Menu;
	
	// Create the new menu with an appropriate title
	pPlayer->m_Menu = new BotMenu(menuID, title, pageLength);
}



/*************************************************
/ POPULATE MENUS
/************************************************/

// MAIN MENU
void PopulateMainMenu(BotMenu * menu)
{
	CBasePlayer * pPlayer;

	menu->AddItem("Add a bot", "addbot");
	menu->AddItem("Command a bot", "command");

	// Add the "(un)pause" name depending on whether the first bot is paused
	pPlayer = FindFirstBot();
	if (pPlayer)
	{
		if (((CBot *)pPlayer)->m_bPaused)
			menu->AddItem("Unpause all bots", "unpause");
		else
			menu->AddItem("Pause all bots", "pause");
	}

	// Add the "(no)debug" name depending on whether the first bot is paused
	if (pPlayer)
	{
		if (((CBot *)pPlayer)->m_bDebugOn)
			menu->AddItem("Stop debugging all bots", "nodebug");
		else
			menu->AddItem("Start debugging all bots", "debug");
	}

	menu->AddItem("Set bot skill level", "skill");
	menu->AddItem("Use the free camera", "freecam");
	menu->AddItem("Use the chase camera", "chasecam");
	menu->AddItem("Remove a bot", "remove");
}

// BOT SELECT MENU
void PopulateAllBots(BotMenu * menu)
{
    for (int i = 1; i <= gpGlobals->maxClients; i++ )
	{
		CBasePlayer * pPlayer = (CBasePlayer *)UTIL_PlayerByIndex( i );
		if(pPlayer && pPlayer->isBot)
		{
			// Add the player's name to the list
			char playerNum[4];
			itoa(i, playerNum, 10);
			menu->AddItem(STRING(pPlayer->pev->netname), playerNum);
		}
	}
}

// POPULATE ALL PLAYERS
void PopulateAllPlayers(BotMenu * menu, CBasePlayer * pIssuingPlayer)
{
    for (int i = 1; i <= gpGlobals->maxClients; i++ )
	{
		CBasePlayer * pPlayer = (CBasePlayer *)UTIL_PlayerByIndex( i );
		
		// Make sure that the player exists and has not been kicked
		if(pPlayer && (pPlayer != pIssuingPlayer) && (pPlayer->pev->takedamage != DAMAGE_NO))
		{
			// Add the player's name to the list
			char playerNum[4];
			itoa(i, playerNum, 10);
			menu->AddItem(STRING(pPlayer->pev->netname), playerNum);
		}
	}
}

// COMMANDS MENU
void PopulateCommandsMenu(BotMenu * menu)
{
	CBot * pBot = (CBot*)UTIL_PlayerByIndex( botBeingCommanded );

	menu->AddItem("Load a behavior DLL", "loaddll");
	if (pBot->m_bPaused)
		menu->AddItem("Unpause the bot", "pause");
	else
		menu->AddItem("Pause the bot", "pause");
	if (pBot->m_bDebugOn)
		menu->AddItem("Stop debugging the bot", "debug");
	else
		menu->AddItem("Start debugging the bot", "debug");
	menu->AddItem("Set skill level", "skill");
}

// SKILL LEVEL MENU
void PopulateSkillLevels(BotMenu * menu)
{
	menu->AddItem("100 - Best", "100");
	menu->AddItem("80", "80");
	menu->AddItem("60", "60");
	menu->AddItem("50", "50");
	menu->AddItem("40", "40");
	menu->AddItem("20", "20");
	menu->AddItem("0 - Worst", "0");
}

// DLL LOAD MENU
void PopulateAvailableDlls(BotMenu * menu)
{
	struct _finddata_t dll_file;
    long hFile;
	char path[1000];

	BuildModFilename(path, "dlls/behavior_dlls/*.dll");

	// Search for all .dll files in the directory
	if( (hFile = _findfirst( path, &dll_file )) == -1L )
		return; // there are no DLL's to load
	else
	{
		menu->AddItem(dll_file.name, dll_file.name);
		while( _findnext( hFile, &dll_file ) == 0 )
		{
			menu->AddItem(dll_file.name, dll_file.name);
		}
	}
}