/**********************************************
/ BotMenu
/
/ A flexible, "page-able" menu system for use
/ with the Half-Life text-menu system.
/
/ Written by Greg Dunham, May 2001
**********************************************/

#ifndef MENU_H
#define MENU_H

#define MAX_ITEMNAME_LENGTH 256
#define MAX_RETURN_LENGTH 256
#define MAX_MENU_ITEMS 256

enum _Menu {
    Menu_Main,
	Menu_Bot_Select,
	Menu_Command,
    Menu_Load_DLL,
	Menu_Remove_Select,
	Menu_Chasecam,
	Menu_Skill,
	Menu_Individual_Skill
};

class BotMenuItem {
// A single menu entry
public:
	BotMenuItem(const char * itemName, const char * returnValue);
	char name[MAX_ITEMNAME_LENGTH];
	char returnVal[MAX_RETURN_LENGTH];
};

class BotMenu {
// Manage all of the menu entries and show the menu
private:
	// Private member variables
	_Menu menuID;		// for setting the current menu when shown
	char menuTitle[256];// title to be displayed at the top of each menu page
	int pageLength;		// how many items to list on a page
	int numItems;		// number of items in the menu
	int currentPage;	// the page number that is currently being shown
	int bitmask;		// which "slots" are active
	BotMenuItem * items[MAX_MENU_ITEMS];

	// Private member functions
	char * MakePageableMenuString();
	void ActivateSlot(int slot);
public:
	// Public member functions
	BotMenu(_Menu ID, const char * title, int pageSize);
	void AddItem(const char * itemName, const char * returnValue);
	void ClearMenu();
	void InitMenu(const char * title, int setPageLength);	// no longer necessary
	void NextPage();
	void ShowMenu(CBasePlayer *pPlayer);
	char * GetReturnValue(int itemIndex);
};

void ShowHlMenu (CBasePlayer *pPlayer, int bitsValidSlots, int nDisplayTime, BOOL fNeedMore, char *pszText);
void GivePlayerNewMenu(CBasePlayer * pPlayer, _Menu menuID, const char * title, int pageLength);

// Menu population
void PopulateMainMenu(BotMenu * menu);
void PopulateAllBots(BotMenu * menu);
void PopulateCommandsMenu(BotMenu * menu);
void PopulateAvailableDlls(BotMenu * menu);
void PopulateAllPlayers(BotMenu * menu, CBasePlayer *);
void PopulateSkillLevels(BotMenu * menu);

#endif

