#include <stdio.h>
#include <winsock.h>
#include "SocketServer.h"

// Helper macro for displaying errors
#define PRINTERROR(s)	\
		fprintf(stderr,"\n%: %d\n", s, WSAGetLastError())

SocketServer::SocketServer(short portToListen) {
	port = portToListen;

	socket_initialized = false;
}


bool SocketServer::Init() {
	// printf("Initializing server listen on port %d...\n", port);

	//
	// Initialize WinSock and check version
	//
	WORD wVersionRequested = MAKEWORD(1,1);
	int nRet = WSAStartup(wVersionRequested, &wsaData);
	if (wsaData.wVersion != wVersionRequested)
	{	
		fprintf(stderr,"\n Wrong version\n");
		return false;
	}
	
	//
	// Create a TCP/IP stream socket to "listen" with
	//
	//SOCKET	listenSocket;

	listenSocket = socket(AF_INET,			// Address family
						  SOCK_STREAM,		// Socket type
						  IPPROTO_TCP);		// Protocol
	if (listenSocket == INVALID_SOCKET)
	{
		PRINTERROR("socket()");
		return false;
	}


	//
	// Fill in the address structure
	//
	//SOCKADDR_IN saServer;		

	saServer.sin_family = AF_INET;
	saServer.sin_addr.s_addr = INADDR_ANY;	// Let WinSock supply address
	saServer.sin_port = htons(port);		// Use port from command line

	//
	// bind the name to the socket
	//

	nRet = bind(listenSocket,				// Socket 
				(LPSOCKADDR)&saServer,		// Our address
				sizeof(struct sockaddr));	// Size of address structure
	if (nRet == SOCKET_ERROR)
	{
		PRINTERROR("bind()");
		closesocket(listenSocket);
		return false;
	}

	//
	// This isn't normally done or required, but in this 
	// example we're printing out where the server is waiting
	// so that you can connect the example client.
	//
	int nLen;
	nLen = sizeof(SOCKADDR);
	char szBuf[256];

	nRet = gethostname(szBuf, sizeof(szBuf));
	if (nRet == SOCKET_ERROR)
	{
		PRINTERROR("gethostname()");
		closesocket(listenSocket);
		return false;
	}

	//
	// Show the server name and port number
	//
	printf("\nServer named %s waiting on port %d\n",
			szBuf, port);

	//
	// Set the socket to listen
	//

	//printf("\nlisten()");
	nRet = listen(listenSocket,					// Bound socket
				  SOMAXCONN);					// Number of connection request queue
	if (nRet == SOCKET_ERROR)
	{
		PRINTERROR("listen()");
		closesocket(listenSocket);
		return false;
	}

	socket_initialized = true;
	return true;
}


void SocketServer::CloseServer() {
	socket_initialized = false;
	closesocket(listenSocket);
	
	// Release WinSock
	WSACleanup();
}


bool SocketServer::ReceiveMessage(char * buffer) {
	SOCKET	remoteSocket;

	// Initialize the server if needed
	if (!socket_initialized) {
		//printf("Trying to initialize socket...\n");
		if (!Init()) {
			printf("Socket Init() seems to have failed...\n");
			return false;
		}
	}

	// Make timeout values work for non-blocking connection accepting
	timeval timeout;
	timeout.tv_sec=0;
	timeout.tv_usec=0;
	
	fd_set conn; // read variable for select() call
	FD_ZERO(&conn); // initialize connection data
	FD_SET(listenSocket, &conn); // get the data from the listen socket
	int nfds = listenSocket; // why is this plus 1? (dunno, but it works)

	// Check if there is an incoming request and connect to it
	int s = select(nfds, &conn, NULL, NULL, &timeout);
	if (s > 0) { 
		remoteSocket = accept(listenSocket, NULL, NULL);
		if (remoteSocket == INVALID_SOCKET)
		{
			PRINTERROR("accept()");
			closesocket(listenSocket);
			return false;
		}
	}
	else
		return false;
	
	// Receive data from the client
	char szBuf[256];
	memset(szBuf, 0, sizeof(szBuf));
	int nRet = recv(remoteSocket, szBuf, sizeof(szBuf),	0);
	if (nRet == INVALID_SOCKET)
	{
		PRINTERROR("recv()");
		closesocket(remoteSocket);
		return false;
	}

	// Copy the received data into the user's buffer
	strcpy(buffer, szBuf);

	// Close the remote socket before exiting
	closesocket(remoteSocket);
	
	return true;
}
