#ifndef BOT_H
#define BOT_H

#include "animation.h"
#include "effects.h"
#include "player.h"
#include "BotUtility.h"
#include "BotControl.h"
#include "Waypoints.h"
#include "BotSurroundings.h"
#include "TacticalAnalysis.h"

#define FIELD_OF_VIEW 120

#define DO_NOTHING 0
#define DLL 1

#define BOT_STOP_SPEED 0
#define BOT_WALK_SPEED 1
#define BOT_RUN_SPEED 2

#define MAX_NUM_WAYPOINTS 100
#define MAX_NUM_INFOPOINTS 100
#define MAX_CLIENTS 32

// Typedefs for checking if a bot can pick up ammo
typedef struct
{
   char *ammo_name;
   char *weapon_name;
   int max_carry;
} ammo_check_t;

// Typedef for waypoint handling
typedef struct
{
	WAYPOINT waypoint;
	// CBeam * pDisplayBeam;
	int beamLocation;
	//int owner;
	//int others;
	bool removed;
} BotWaypoint;


typedef struct
{
	CBeam * pDisplayBeam;
	int numUsers;
} WaypointBeamMarker;

//infopoints will be used as short lived waypoints
typedef struct
{
	WAYPOINT waypoint;
	//CBeam * pDisplayBeam;
	int startTime;
	int lifeTime;
	int beamLocation;
	int owner;
	bool removed;
} BotInfopoint;

//int numInfopointBeams;
//WaypointBeamMarker InfopointBeams[MAX_NUM_INFOPOINTS * MAX_CLIENTS];

// Typedef for function pointer to external think routine
typedef int	(* PFN_CREATE_BOT_BEHAVIOR)();
typedef void (* PFN_REMOVE_BOT_BEHAVIOR)(int);
typedef void (* DO_BOT_THINK)(int, int);
typedef int (* GET_ACTION_INTEGER)(int);
typedef bool (* GET_ACTION_BOOLEAN)(int);

// Vision-checking result type returned by GetVisionDistance()
typedef struct
{
	int vdistance;
	char ent_class[40];

} VisionResult;

void BotCreate( void );		// create a new bot
void BotDebug (char *msg);	// dump debug messages to the console
 

class CBot : public CBasePlayer
// The bot class inherits many functions from CBasePlayer class
{
public:
	// STATE INFORMATION
	bool m_bPaused;					// is the bot paused?
 	int playerNum;					// the bot's client index
	BotSurroundings surroundings;	// inventory of nearby items
	unsigned short int aimSkill;	// accuracy of aiming (0-100, 100=perfect, 0=terrible)
	float percentTurnError;			// how much turning-error is being applied?
	float nextAimErrorChange;		// for checking if it is time to recalculate
	float player_look_error[MAX_CLIENTS];		// how much error is currently being applied
	float last_player_look_time[MAX_CLIENTS];	// last time the player was checked
	float xError;
	float yError;
	float zError;
	void CheckStatChanges();
	char behaviorName[256];
	UpdateFlexMonitor();			// sends the bot's state information to FlexMonitor modules

	//WAYPOINT AND INFOPOINT SUPPORT
	// waypoints
	BotWaypoint waypoints[MAX_NUM_WAYPOINTS];
	int m_nWaypoints;
	void AddWaypoint(WAYPOINT); // add a waypoint to the bot's waypoint container
	void DelWaypoint(BotWaypoint); // delete a waypoint from the bot's waypoint container
	void ExportWaypointsToFile(const char * filename);
	void ReadWaypointsFromFile(const char * filename);
	// infopoints
	void AddInfopoint(WAYPOINT waypoint, int startingTime, int lifeTime);
	void expireInfopoint(BotInfopoint);
	void checkInfopoints();
	BotInfopoint Infopoints[MAX_NUM_INFOPOINTS];
	int m_nInfopoints;

	
	//BACKWARD COMPATABILITY
	//this memeber variable is to specify weather to use the
	//old noncoherant angle assignments or the new correlated angle
	//assignments, the new corelated assignmnets mean that all
	//yaw and turn sensors/actuators use a standard sign convention
	//+ numbers =  to the right 
	//- numbers = to the left 
	//all pitch values use + numbers for up and - numbers for down
	//this variable is to be se in the dllmain() function of a behavior
	//seting this varable to true means that the new sign convention is used 
	//by default this variable is set to false so that old dlls will get the 
	//sign values that were expected at the time of their creeation.
	bool newTurnSignsP;

	// ACTUATORS
	void do_shoot(bool);
	void do_jump(bool);
	void do_duck(bool);
	void do_pitch(int speed);
	void do_strafe(int speed);
	void do_translation(int speed);
	void do_rotation(int speed);
	void do_secondary(bool);
	void do_weapon(int);
	void do_reload(bool);
	void do_use(bool);
	void add_waypoint();
	void add_infopoint(int lifespanMs, int type);
	void comms_set_int(int, int);
	void comms_set_float(int, float);
	void comms_set_bool(int, bool);

	// SENSORS
	int get_health();
	float get_yaw();
	float get_pitch();
	bool IsEntityVisible(CBaseEntity * pEntity);
	bool IsLocationVisible(Vector Location);
	bool is_player_visible(int playerNum);
	float yaw_to_player(int playerNum);
	float pitch_to_player(int lookPlayerNum);
	int distance_to_player(int lookPlayerNum);
	VisionResult get_vision_info(float lookAngle, int checkDistance, CBeam * pBeam);
	VisionResult get_vision_info(float lookAngle, float checkPitch, int checkDistance, CBeam * pBeam);
	VisionResult get_sweep_info(float lookAngle, int checkDistance, int sweep, int interval);
	int get_vision_distance(float lookAngle, int checkDistance);
	int get_vision_distance(float lookAngle, float pitch, int checkDistance);
	int get_vision_sweep_distance(float lookAngle, int checkDistance, int sweep, int interval);
	int get_left_distance(int checkDistance);
	int get_right_distance(int checkDistance);
	int get_center_distance(int checkDistance);
	float YawToEntity(CBaseEntity * pEnt);
	float YawToLocation(Vector location);
	int DistanceToEntity(CBaseEntity * pEnt);
	int DistanceToLocation(Vector location);
	float PitchToEntity(CBaseEntity * pEnt);
	float PitchToLocation(Vector location);
	int nearest_vis_enemy_num();
	bool is_moving();
	float yaw_to_waypoint(int waypoint_index);
	int distance_to_waypoint(int waypoint_index);
	int nearest_waypoint_num();
	float yaw_to_infopoint(int infopoint_index, int type);
	int distance_to_infopoint(int infopoint_index, int type);
	int next_ip_to_expire_num(int type);
	int nearest_infopoint_num(int type);
	int ammo_in_clip();
	int nearest_vis_teammate_num();
	float yaw_to_enemy_sound();
	int distance_to_enemy_sound();
	int nearest_item(int itemType);
	float yaw_to_item(int itemType, int index);
	int distance_to_item(int itemType, int index);
	float pitch_to_item(int itemType, int index);
	bool can_get_item(int itemType, int itemIndex);
	int get_x();
	int get_y();
	int get_z();
	int get_current_weapon();
	bool is_friend(int player_index);
	float pitch_to_infopoint(int infopointType, int index);
	float pitch_to_waypoint(int index);
	bool get_possesses_gun(int gun);
	int primary_ammo_in_stock(int gunID);
	int secondary_ammo_in_stock(int gunID);
	float CalculateSensorError(int playerNum);
	int player_num();
	int comms_get_int(int, int);
	float comms_get_float(int, int);
	bool comms_get_bool(int, int);
	//bool is_on_ladder();


	// DEBUGGING
	bool m_bDebugOn;			// should the bot send out debug messages?
	bool m_bAutoSensorDebug;	// should the sensors automatically send debug info?

	// BEHAVIOR LOADING
	HINSTANCE m_hInstDll;		// a handle to the DLL that is loaded
	int m_behaviorInstance;		// the instance of this bot in the DLL space
	bool LoadBehaviorDll(const char * dllFilename); // load a new behavior DLL
	void UnloadBehaviorDll();
	PFN_CREATE_BOT_BEHAVIOR m_pfnCreateBotBehavior;
	PFN_REMOVE_BOT_BEHAVIOR m_pfnRemoveBotBehavior;
	
	DO_BOT_THINK m_pfnDoExternalBotThink; // pointer to the think function
	GET_ACTION_INTEGER m_pfnGetExternalRotate;
	GET_ACTION_INTEGER m_pfnGetExternalTranslate;
	GET_ACTION_INTEGER m_pfnGetExternalStrafe;
	GET_ACTION_INTEGER m_pfnGetExternalPitch;
	GET_ACTION_BOOLEAN m_pfnGetExternalShoot;
	GET_ACTION_BOOLEAN m_pfnGetExternalJump;
	GET_ACTION_BOOLEAN m_pfnGetExternalDuck;
	GET_ACTION_BOOLEAN m_pfnGetExternalSecondary;
	GET_ACTION_INTEGER m_pfnGetExternalWeapon;
	GET_ACTION_BOOLEAN m_pfnGetExternalReload;
	GET_ACTION_BOOLEAN m_pfnGetExternalUse;
	

	// Performance metrics
	unsigned long totalThinkTime;
	unsigned int totalThinks;


	/* BOT FITNESS STATISTICS
	int statKills;
	int statFriendlyKills;
	int statDeaths;
	int statSuicides;
	float statTimeMoving;
	float statTimeMovingTotal;*/
	int get_stat_kills();
	int get_stat_friendly_kills();
	int get_stat_deaths();
	int get_stat_suicides();
	float get_stat_percent_time_moving();	// not yet implemented!

	/*/ Last stat values (for stat change notification)
	int lastKills;
	int lastFriendlyKills;
	int lastDeaths;
	int lastSuicides;
*/
	// UTILITY
	Vector CBot::GetGunPosition();
	//unsigned int CBot::friends(int bot_num);


	// ENGINE CALLS
	void Spawn( void );
	void Killed( entvars_t *pevAttacker, int iGib ); //called when bot is killed
	void BotThink( void ); 
    void PlayerDeathThink( void ); 
	virtual BOOL IsNetClient( void ) { return FALSE; }
    int BloodColor() { return BLOOD_COLOR_RED; }
	void SetActivity(int act);
	int  GetActivity() { return m_Activity; }
	int m_Activity;
	CBot();

private:
	// STATE
	Vector v_prev_origin; 	// previous location
	int botTask;			// what should the bot be doing?
	double lastThinkTime;	// stored in seconds
	float findItemsTime;	// time for next item search
	
	// UTILITY
	bool CheckCommonErrorConditions();
	void HandleBotCommands();
	void LookForItems();
	long start_time_ms;

	// ACTUATORS
	float f_strafe_speed;
	float f_move_speed;

	// SENSORS
	int ClosestSurroundingItem(BotSurroundingItems * itemInventory);
	int ClosestSurroundingUnstableItem(BotSurroundingItems * itemInventory);
	int ClosestSurroundingLadderItem(BotSurroundingItems * itemInventory);

	// DEBUGGING
	CBeam * m_pBeam;
	char m_debugStr[256];	// use this for creating debug message strings
	bool m_bUpdatingCenterBeam;
	CBeam * m_pCenterBeam;
	bool m_bUpdatingRightBeam;
	CBeam * m_pRightBeam;
	bool m_bUpdatingLeftBeam;
	CBeam * m_pLeftBeam;
	CBeam * InitNewDistanceBeam(int rColor, int gColor, int bColor);
	void DoDebuggingBeams();
	void CleanupDebugBeams();
};

void CompleteBackloggedTasks();

// The following functions SHOULD be in BotUtility.cpp, but causes include problems!
// *** FIX THIS SOMETIME!!! *** //
CBot * FindFirstBot();
int GetPlayerIndex(CBasePlayer * player);
void KickPlayer(int playerNum);
int getWeaponIndex(CBasePlayerItem * weapon);

// The following functions SHOULD be in TacticalAnalysis.cpp, but causes include problems!
// *** FIX THIS SOMETIME!!! *** //
_TacticalHeightType TacticalHeightDifference(CBasePlayer * pMainPlayer, 
										 CBasePlayer * pTargetPlayer);

_TacticalDistanceType TacticalDistance(CBasePlayer * pMainPlayer, 
								 CBasePlayer * pTargetPlayer);

#endif //BOT_H
